<?PHP
###########################################################################
# 
# File:            taloLoggerGraph.inc (PHP)
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Usage:           To use the taloLoggerGraph log graphing package in 
#                  your website, first make sure that the files are
#                  located in your PHP-include-path.
#
#                  Secondly generate a configuration file for
#                  taloLoggerGraph. (see taloLoggerGraph.conf example
#                  configuration file and http://olammi.iki.fi/sw/taloLogger/
#                  for help).
#
#                  Utilise the sample index.php or add the following code to 
#                  your own web site code:
#
#                          include("taloLoggerGraph.inc");
#
#                          taloLoggerGraph("taloLoggerGraph.conf");
#                      
#                  Replace the taloLoggerGraph.conf configuration file name with
#                  the name of the configuration file you generated. Remember
#                  to enclose the code in PHP-tags (see index.php for example).
#
#                  Executing from the command line to produce data to the
#                  file system:
#
#                    php taloLoggerGraph.inc [options] <conf-file> <output-dir> 
#
#                  Available options:
#                    
#                    -d <date>     - Date for the result (defaults to current
#                                    date). Date format: yyyy-mm-dd
#                                    (example: -d 2014-08-25)
#
#                    -p <period>   - If date is not given, period in hours.
#                                    (example: -p 24)
# 
#                    --prefile <file> - File to add in beginning of result
#                                    index.html file.
#
#                    --postfile <file> - File to add in the end of result
#                                    index.html file.
#
#                    --regenerateCache -  if given, cached images for the date will be 
#                                    regenerated (if cache is active)
#
#                    --drawMethod <id> -  use this drawing method regardless of
#                                    configuration file setting:
#                                      0 - Default, use conf file setting
#                                      1 - Use server side images (JpGraph)
#                                      2 - Use client side javascript (d3js)
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.2b
#
# Date:            09.01.2015
#
# Functions:       taloLoggerGraph
#                   
# Description:     House automation data logger graphing application for PHP. 
# 
#                  The main function.
#
# Requirements:    PHP 5.1 or newer (www.php.net)
#
#                  Suitable PDO database support for PHP.
#
#                  JpGraph PHP-graphing tool (http://jpgraph.net/)
#                  (Optionally uses Web Browser and d3js javascript
#                  library for drawing graphs.)
#
# Version history: ** 20.10.2010 v0.1a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 21.10.2010 v0.1b (Olli Lammi) **
#                  Small bug fixes. Database optimisations.
#
#                  ** 23.10.2010 v0.1c (Olli Lammi) **
#                  SQLite version.
#
#                  ** 03.11.2010 v0.1d (Olli Lammi) **
#                  Legend changes.
#
#                  ** 04.11.2010 v1.0a (Olli Lammi) **
#                  First version released to the public.
#
#                  ** 05.11.2010 v1.0b (Olli Lammi) **
#                  Bug fixes and support for non PDO 
#                  database connection for SQLite.
#
#                  ** 15.11.2010 v1.0c (Olli Lammi) **
#                  Support for IMAGE type graphs.
#
#                  ** 15.11.2010 v1.0d (Olli Lammi) **
#                  Added feature to hide the calendar.
#
#                  ** 16.11.2010 v1.0e (Olli Lammi) **
#                  Chart image size in parameter.
#
#                  ** 15.01.2013 v1.1a (Olli Lammi) **
#                  Support for taloLogger native DB structure.
#
#                  ** 05.02.2013 v1.1b (Olli Lammi) **
#                  Running last 24h of log instead of one day.
#
#                  ** 15.05.2014 v1.1c (Olli Lammi) **
#                  Possibility to configure more than 2 Y-axises. 
#                  Automatic scaling for axises.
#
#                  ** 05.09.2014 v1.1c (Olli Lammi) **
#                  When run from command line, script will output
#                  to a directory with no calendar. 
#
#                  ** 01.12.2014 v1.1c (Olli Lammi) **
#                  Image caching features.
#
#                  ** 10.12.2014 v1.1d (Olli Lammi) **
#                  Fixed leaking array references from code.
#                  Fixes to sample configuration.
#
#                  ** 31.12.2014 v1.1e (Olli Lammi) **
#                  Fixed caching problem if current date is selected
#                  from calendar.
#
#                  ** 02.01.2015 v1.2a (Olli Lammi) **
#                  Test version to also support graphing in the browser.
#
#                  ** 09.01.2015 v1.2b (Olli Lammi) **
#                  Javascript graphing support released. Minor bug
#                  fixes and performance enhancement. 
#
###########################################################################

include('taloLoggerGraph_functions.inc');

function taloLoggerGraph($confname) {
    $date = '';
    if (array_key_exists('date',$_REQUEST)) $date = $_REQUEST['date'];
    $period = 0; 
    if (array_key_exists('period',$_REQUEST)) $period = $_REQUEST['period'];
    $regenerateCache = false;
    if (array_key_exists('regenerateCache',$_REQUEST) && $_REQUEST['regenerateCache'] == 'true') $regenerateCache = true;
    $forceDrawingMethod = 0;
    if (array_key_exists('forceDrawingMethod',$_REQUEST)) $forceDrawingMethod = $_REQUEST['forceDrawingMethod'];
    
    $resstr = taloLoggerGraphImpl($date, $period, $confname, FALSE, '', $regenerateCache, $forceDrawingMethod);

    echo $resstr;
}

function taloLoggerGraphImpl($date, $period, $confname, $isCmdline, $outputdir, $regenerateCache, $forceDrawingMethod) {
    umask(0002);

    $resstr = '';

    $config = readConfigurationFile($confname);
    if ($config['stat'] != 0) {
        output_error($config['msg']);
        exit();
    }
    $config = $config['result'];

    $hidecal = 0;
    if ($isCmdline) {
        $hidecal = 1;
    }
    if (array_key_exists('HIDE_CALENDAR', $config['GENERAL'])) {
    	if (strtoupper($config['GENERAL']['HIDE_CALENDAR']) == 'TRUE') {
    		$hidecal = 1;
    	}
    }
    
    $DB_TYPE = $config["GENERAL"]["DB_TYPE"];
    if ($DB_TYPE == 'MYSQL') {
        $LOG_HOST = $config["GENERAL"]["MYSQLDB_HOST"];
        $LOG_USER = $config["GENERAL"]["MYSQLDB_USER"];
        $LOG_PASSWD = $config["GENERAL"]["MYSQLDB_PASSWD"];
        $LOG_DBNAME = $config["GENERAL"]["MYSQLDB_NAME"];
    }
    else if ($DB_TYPE == 'SQLITE') {
        $LOG_DBNAME = $config["GENERAL"]["SQLITEDB_NAME"];
    }
    else {
        output_error("ERROR: Invalid database type in configuration: " . $DB_TYPE);
        exit();        
    }
    
    $dbnative = 0;
    if (array_key_exists('DB_TALOLOGGER_SCHEMA_VERSION', $config['GENERAL'])) {
    	if ($config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] == '1') {
    		$dbnative = 1;
    	}
    	else if ($config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] == '0') {
    		$dbnative = 0;
    	}
    	else {
        	output_error("ERROR: Invalid database native schema version in configuration.");
        	exit();        
    	}
    }
        			
    if ($dbnative == 1) {
    	$config['GENERAL']['DB_TABLE'] = 'talo_data';
        $config['GENERAL']['DB_TIMECOL'] = 'time';
        $config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] = 1;
    }
    else {
        $config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] = 0;
    }
    
    $LOG_TABLE = $config["GENERAL"]["DB_TABLE"];
    $LOG_TIMECOL = $config["GENERAL"]["DB_TIMECOL"];
 
    $conn = null;
    if ($DB_TYPE == 'MYSQL') {
        $conn = new MySQLDBConnection($LOG_HOST, $LOG_DBNAME, $LOG_USER, $LOG_PASSWD);
    } 
    else if ($DB_TYPE == 'SQLITE') {
        $conn = new SQLiteDBConnection($LOG_DBNAME);
    }
           
    if (!$conn || !$conn->isOpen()) {
        output_error("ERROR: Unable to open/connect database.");
        exit();        
    }
  
    $today = strftime('%Y-%m-%d');
    $t_vuosi = strftime('%Y');
    $t_kuukausi = strftime('%m');
    $t_paiva = strftime('%d');

    $d_vuosi = $t_vuosi;
    $d_kuukausi = $t_kuukausi;
    $d_paiva = $t_paiva;
    $is_tanaan = 1;
    $is_full_day = 1;

    if ($date && $date == $today) {
    	$is_tanaan = 1;
    }
    else if ($date && preg_match("/^(?P<vuosi>\d+)-(?P<kuukausi>\d+)-(?P<paiva>\d+)$/", $date, $regmatch)) {
        $d_vuosi = $regmatch['vuosi'];
        $d_kuukausi = sprintf("%02d", $regmatch['kuukausi']);
        $d_paiva = sprintf("%02d", $regmatch['paiva']);
        $is_tanaan = 0;
    }
    else if ($date && intval($date) != 0) {
    	$temptime = time() + intval($date) * (24 * 60 * 60);
    	$d_vuosi = strftime('%Y', $temptime);
    	$d_kuukausi = strftime('%m', $temptime);
    	$d_paiva = strftime('%d', $temptime);
    	$is_tanaan = 0;
    }

    if (!$hidecal) {
	    $vuodet = array();
	    if ($DB_TYPE == 'MYSQL') {
	        $sqlstmt = "SELECT YEAR(MIN(" . $LOG_TIMECOL . ")) AS ymin, " .
	                   "YEAR(MAX(" . $LOG_TIMECOL . ")) AS ymax FROM " .
	                   $LOG_TABLE;
	    }
	    else if ($DB_TYPE == 'SQLITE') {
	        $sqlstmt = "SELECT ymin, ymax FROM " .
	                   "(SELECT STRFTIME('%Y', DATETIME(" . $LOG_TIMECOL . ", 'localtime')) AS ymin FROM " . 
	                   $LOG_TABLE . " WHERE " . $LOG_TIMECOL . "=(SELECT MIN(". $LOG_TIMECOL .") FROM " . $LOG_TABLE. ") LIMIT 1), " .
	                   "(SELECT STRFTIME('%Y', DATETIME(" . $LOG_TIMECOL . ", 'localtime')) AS ymax FROM " . 
	                   $LOG_TABLE . " WHERE " . $LOG_TIMECOL . "=(SELECT MAX(". $LOG_TIMECOL .") FROM " . $LOG_TABLE. ") LIMIT 1)";
	    }
	    $conn->query($sqlstmt);
	    if($row = $conn->fetchRow()) {
	        $ymin = $row["ymin"];
	        $ymax = $row["ymax"];
	    }
	    else {
	        output_error("ERROR: No data in database.");
	        exit();        
	    }
	    $conn->freeResult();
	    
	    if ($ymin) $vuodet[] = $ymin;
	    if ($ymax - $ymin > 1) {
	        $y = $ymin + 1;
	        while ($y < $ymax) {
	            $sqlstmt = "SELECT " . $LOG_TIMECOL . 
	                       " FROM " . $LOG_TABLE . " WHERE " . 
	                       yearEhto($DB_TYPE, $LOG_TIMECOL, $y) . " LIMIT 1";
	            $conn->query($sqlstmt);
	            if ($conn->fetchRow()) {
	                $vuodet[] = $y;
	            }
	            $conn->freeResult();
	 
	            $y = $y + 1;
	        }
	    }
	    if ($ymax > $ymin) $vuodet[] = $ymax;
	
	    $kuukaudet = array();    
	    $kk = 1;
	    while ($kk <= 12) {
	        $sqlstmt = "SELECT " . $LOG_TIMECOL . 
	                   " FROM " . $LOG_TABLE . " WHERE " . 
	                   monthEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $kk) . " LIMIT 1";
	        $conn->query($sqlstmt);
	        if ($conn->fetchRow()) {
	            $kuukaudet[] = $kk;
	        }
	        $conn->freeResult();
	
	        $kk = $kk + 1;
	    }
	
	    $paivat = array();
	    if ($DB_TYPE == 'MYSQL') {
	        $sqlstmt = "SELECT DAY(MIN(" . $LOG_TIMECOL . ")) AS dmin, " .
	                   "DAY(MAX(" . $LOG_TIMECOL . ")) AS dmax FROM " .
	                   $LOG_TABLE . " WHERE " . 
	                   monthEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi);
	    }
	    else if ($DB_TYPE == 'SQLITE') {           
	        $sqlstmt = "SELECT STRFTIME('%d', DATETIME(MIN(" . $LOG_TIMECOL . "), 'localtime')) AS dmin, " .
	                   "STRFTIME('%d', DATETIME(MAX(" . $LOG_TIMECOL . "), 'localtime')) AS dmax FROM " .
	                   $LOG_TABLE . " WHERE " . 
	                   monthEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi);
	    }
	    $conn->query($sqlstmt);
	    if($row = $conn->fetchRow()) {
	        $dmin = $row["dmin"];
	        $dmax = $row["dmax"];
	    }
	    $conn->freeResult();
	
	    if ($dmin) $paivat[] = $dmin;
	    if ($dmax - $dmin > 1) {
	        $d = $dmin + 1;
	        while ($d < $dmax) {
	            $sqlstmt = "SELECT " . $LOG_TIMECOL . 
	                       " FROM " . $LOG_TABLE . " WHERE " . 
	                       dayEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi, $d) . 
	                       " LIMIT 1";
	            $conn->query($sqlstmt);
	            if ($conn->fetchRow()) {
	                $paivat[] = $d;
	            }
	            $conn->freeResult();
	
	            $d = $d + 1;
	        }
	    }
	    if ($dmax > $dmin) $paivat[] = $dmax;

            $resstr .= '<TABLE BORDER=1 ALIGN="center" WIDTH="100%">
<TR><TD ALIGN=center VALIGN=middle COLSPAN=3>
<A HREF="?date=&period=24">Last 24h</A>
<A HREF="?date=">Today</A>
</TD></TR>
<TR>
<TD WIDTH="33%" ALIGN=center VALIGN=middle>
<TABLE BORDER=0>
<TR>';

	    $item = 0;
	    foreach ($vuodet as $vuosi) {
	        $modifier = '';
	        if ($vuosi == $d_vuosi) {
	            $modifier = " BGCOLOR=grey";
	        }
	
	        $resstr .= "<TD ALIGN=center" . $modifier . ">" 
	             . "<A HREF=\"?date=" . $vuosi . "-" . $d_kuukausi . "-" 
	             . $d_paiva . "\">" . $vuosi . "</A></TD>";
	        $item = ($item + 1) % 3;
	        if ($item == 0) $resstr .= "</TR><TR>";
	    }    

            $resstr .= '</TR>
</TABLE>
</TD>
<TD WIDTH="33%" ALIGN=center VALIGN=middle>
<TABLE BORDER=0>
<TR>';

	    $item = 0;
	    foreach ($kuukaudet as $kuukausi) {
	        $modifier = '';
	        if ($kuukausi == $d_kuukausi) {
	            $modifier = " BGCOLOR=grey";
	        }
	
	        $resstr .= "<TD ALIGN=center" . $modifier . ">" 
	             . "<A HREF=\"?date=" . $d_vuosi . "-" . sprintf("%02d", $kuukausi) . "-" 
	             . $d_paiva . "\">" . $kuukausi . "</A></TD>";
	        $item = ($item + 1) % 6;
	        if ($item == 0) $resstr .= "</TR><TR>";
	    }
        $resstr .= "</TR>\n</TABLE>\n</TD>\n" 
                . "<TD WIDTH=\"33%\" ALIGN=center VALIGN=middle>\n"
                . "<TABLE BORDER=0>\n";

	    if (count($paivat) > 0) {
	        $resstr .= "<TR>";
	        $minpaiva = $paivat[0];
	        $maxpaiva = $paivat[ count($paivat) - 1 ];
	        $vkonpaiva = (date('w', mktime(0, 0, 0, $d_kuukausi, $minpaiva, $d_vuosi)) + 6) % 7;
	
	        if ($vkonpaiva > 0) {
	            foreach( range(1, $vkonpaiva) as $i ) {
	                $resstr .= "<TD>&nbsp;</TD>";
	            }
	        }
	
	        foreach( range($minpaiva, $maxpaiva) as $paiva ) {
	            if ( in_array($paiva, $paivat) ) {
	                $modifier = '';
	                if ($paiva == $d_paiva) {
	                    $modifier = " BGCOLOR=grey";
	                }
	
	                $resstr .= "<TD ALIGN=center" . $modifier . ">" 
	                     . "<A HREF=\"?date=" . $d_vuosi . "-" . $d_kuukausi . "-" 
	                     . sprintf("%02d", $paiva) . "\">" . $paiva . "</A></TD>";
	            }
	            else {
	                $resstr .= "<TD>&nbsp;</TD>";
	            }
	
	            $vkonpaiva = ($vkonpaiva + 1) % 7;
	            if ($vkonpaiva == 0) {
	                $resstr .= "</TR><TR>";
	            }       
	        }
	
	        while ($vkonpaiva > 0) {
	            $vkonpaiva = ($vkonpaiva + 1) % 7;
	            $resstr .= "<TD>&nbsp;</TD>";
	        }
	
	        $resstr .= "</TR>\n";
	    }

            $resstr .= "</TABLE>\n</TD>\n</TR>\n</TABLE>\n";
        }
    
	$has_data = 0;
	$starttime = 0;
	$endtime = 0;
	$calctime = time();
	if ($is_tanaan && $period > 0) {
		$endtime = nextFullHour($calctime);
		$starttime = $endtime - ($period * 3600);
		$is_full_day = 0;
	}
	else {		
	    $starttime = mktime(0, 0, 0, $d_kuukausi, $d_paiva, $d_vuosi);		
		$endtime = mktime(23, 59, 59, $d_kuukausi, $d_paiva, $d_vuosi) + 1;
	}

    $sqlstmt = "SELECT " . $LOG_TIMECOL .
               " FROM " . $LOG_TABLE . " WHERE ";
    if ($starttime > 0) {
    	$sqlstmt .= uxtimeEhto($DB_TYPE, $LOG_TIMECOL, $starttime, $endtime);
    }
    else {
    	$sqlstmt .= dayEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi, $d_paiva); 
    }
	$sqlstmt .= " LIMIT 1";
    $conn->query($sqlstmt);
    if ( $conn->fetchRow() ) {
        $has_data = 1;
    }
    $conn->freeResult();

    if ($is_full_day) {
        $resstr .= "<P><DIV ALIGN=center>" . strftime('%d.%m.%Y', $starttime) . "<P>";
    }
    else {
    	$resstr .= "<P><DIV ALIGN=center>" . $period . " hours / " . getTimeStr($calctime) . "<P>";
    }
    
    $js_charts = 0;
    if (array_key_exists('JAVASCRIPT_CHARTS', $config['GENERAL'])) {
    	if (strtoupper($config['GENERAL']['JAVASCRIPT_CHARTS']) == 'TRUE') {
    		$js_charts = 1;
    	}
    }
    
    if ($forceDrawingMethod == "1") {
    	$js_charts = 0;
    }
    else if ($forceDrawingMethod == "2") {
    	$js_charts = 1;
    }
    
    $has_cached = false;    
    if ($has_data > 0) {
        $ids = generateLoggerGraphs($DB_TYPE, $conn, $config, $is_tanaan, $is_full_day, $starttime, $endtime, $calctime, $regenerateCache, $js_charts, $isCmdline);

        foreach($ids as $ida) {
        	if ($ida['type'] == 'html') {
        		$resstr .= getHtmlChartPreDocument($isCmdline);
        		break;
        	}
        }
        
        $fileid = 0;
        foreach($ids as $ida) {
        	$id = $ida['id'];
        	$type = $ida['type'];
        	$graphcontent = $ida['content'];
        	$extrafiles = $ida['files'];
        	
        	$temphtml = null;
        	
        	# check for cached image (id starts with cache_)
        	$cached_image = false;
        	if (substr($id, 0, 6) === "cache_") {
        		$cached_image = true;
        		$has_cached = true;
        		$id = substr($id, 6);
        	}
      	         	
        	if ($type == 'html') {
        		if ($cached_image) {
        			$fname = $config['GENERAL']['CACHE_DIRECTORY'] . '/' . $id;
        			$temphtml = file_get_contents($fname);
        		}
        		else {
        			$temphtml = $graphcontent;
        		}        		
        	}
        	else if ($type == 'png') {
	            if ($isCmdline) {
	            	if ($cached_image) {
	            		$fname = $config['GENERAL']['CACHE_DIRECTORY'] . '/' . $id;
	            	}
	            	else {
	                	$fname = getImagePath($id, $type);
	            	}
	            	$fileid++;	            	
	                $outfname = 'image_' . $fileid . '.' . $type;
	                if (file_exists($fname)) {
	                    copy($fname, $outputdir . '/' . $outfname);
						if (!$cached_image) {
							unlink($fname);
						}                    
						$temphtml = '<IMG SRC="' . $outfname . '"><BR>';						
	                }
	            }
	            else {
	            	if ($cached_image) {
	            		$temphtml = '<IMG SRC="' . $config['GENERAL']['CACHE_URL_PATH'] . '/' . $id . '"><BR>';
	            	}
	            	else {            	
	                	$temphtml = '<IMG SRC="taloLoggerGraph_getchart.php?img=' . $id . '"><BR>';
	            	}      
	            }
        	}
        	else {
        		$temphtml = '<I>ERROR: Unknown graph part type.</I><BR><BR>';
        	}
        	
        	if ($extrafiles && count($extrafiles) > 0) {
        		foreach($extrafiles as $efile) {
        			if ($isCmdline) {
        				$fileid++;
        				$outfname = 'image_' . $fileid . '.' . $efile['type'];
        				if ($temphtml && $efile['url']) {
        					$temphtml = str_replace($efile['url'], $outfname, $temphtml);
        				}
        				copy($efile['fname'], $outputdir . '/' . $outfname);
        			}
        			else {        			
        				if ($temphtml && $efile['url']) {
        					$imgurl = findURLForFile($efile['fname']);
        					$temphtml = str_replace($efile['url'], $imgurl, $temphtml);
        				}        				
        			}
        		}
        	}
        	         	
        	if ($temphtml) {        		
        		$resstr .= $temphtml;
        	}
        }
    }
    else {
        $resstr .= "<I>No data in the database for requested date.</I>";
    }
    
    if ($has_cached && !$isCmdline) {
    	$tempstr = '?regenerateCache=true';
    	if ($period) $tempstr .= '&period=' . $period;    	 
    	if ($date) $tempstr .= '&date=' . $date;
    	$resstr .= 'Contains cached images. <A HREF="' . $tempstr . '">Click here to refresh cache for this page.</A>';
    }

    if (!$isCmdline) {
        $infostr = '';
        $tempstr = '?forceDrawingMethod=';
        if ($js_charts) {
            $tempstr .= '1';
            $infostr = 'View as server generated images';
        }
        else {
            $tempstr .= '2';
            $infostr = 'View as client side javascript';
        }
    	if ($period) $tempstr .= '&period=' . $period;    	 
    	if ($date) $tempstr .= '&date=' . $date;
    	$resstr .= '<BR><A HREF="' . $tempstr . '">' . $infostr . '</A>';
    }
    
    $resstr .= "</DIV>";

    $conn->close();
    $conn = null;

    return $resstr;
}

function cli_usage() {
    echo "USAGE: php taloLoggerGraph.inc [options] <conf-file> <output-dir>\n";
    echo "\n";
    echo "Options:\n";
    echo "  -d <date>         -  yyyy-mm-dd or number for days forward/back, eg. -1 means yesterday\n";
    echo "  -p <period>       -  hours\n";
    echo "  --prefile <file>  -  file to add in front of index.html\n"; 
    echo "  --postfile <file> -  file to add to end of index.html\n";
    echo "  --regenerateCache -  if given, cached images for the date will be regenerated\n";
    echo "                       (if cache is active)\n";
    echo "  --drawMethod <id> -  use this drawing method regardless of\n";
    echo "                       configuration file setting:\n";
    echo "                          0 - Default, use conf file setting\n";
    echo "                          1 - Use server side images (JpGraph)\n";
    echo "                          2 - Use client side javascript (d3js)\n";
    exit();
}

# command line execution
if (php_sapi_name() == 'cli') {
    echo "taloLoggerGraph " . getVersion() . "\n";

    $prefixfile = '';
    $postfixfile = '';
    $date = '';
    $period = '';
    $regenerateCache = false;
    $forceDrawingMethod = "0";

    if (count($argv) < 3) cli_usage();

    $options = getopt("d:p:", array("prefile:", "postfile:", "drawMethod:", "regenerateCache"));

    if (array_key_exists('d', $options)) $date = $options['d'];
    if (array_key_exists('p', $options)) $period = $options['p'];
    if (array_key_exists('prefile', $options)) $prefixfile = $options['prefile'];
    if (array_key_exists('postfile', $options)) $postfixfile = $options['postfile'];
    if (array_key_exists('regenerateCache', $options)) $regenerateCache = true;
    if (array_key_exists('drawMethod', $options)) $forceDrawingMethod = $options['drawMethod'];
    
    $conffile = $argv[count($argv) - 2];
    $outputdir = $argv[count($argv) - 1];

    if (!file_exists($conffile)) {
        echo "ERROR: Configuration file: " . $conffile . " not found.\n";
        cli_usage();
    }

    if (!file_exists($outputdir)) {
        echo "Creating output directory: " . $outputdir . "\n";
        mkdir($outputdir);
    }

    if ($prefixfile != '' && !file_exists($prefixfile)) {
        echo "ERROR: Pre-file: " . $prefixfile . " not found.\n";
        cli_usage();
    }

    if ($postfixfile != '' && !file_exists($postfixfile)) {
        echo "ERROR: Post-file: " . $postfixfile . " not found.\n";
        cli_usage();
    }

    if (!in_array($forceDrawingMethod, array('0','1','2'))) {
        echo "ERROR: Invalid drawMethod.\n";
        cli_usage();
    }

    $resstr = taloLoggerGraphImpl($date, $period, $conffile, TRUE, $outputdir, $regenerateCache, $forceDrawingMethod);

    if ($prefixfile == '' && $postfixfile == '') {
        $resstr = "<HTML><BODY BGCOLOR=white>\n" . $resstr . "\n</BODY></HTML>\n";
    }
    else {
        $prestr = '';
        if (file_exists($prefixfile)) {
            $prestr = file_get_contents($prefixfile);
        }

        $poststr = '';
        if (file_exists($postfixfile)) {
            $poststr = file_get_contents($postfixfile);
        }

        $resstr = $prestr . $resstr . "\n" . $poststr;
    }

    $outfilename = $outputdir . '/index.html';
    file_put_contents($outfilename, $resstr);
}
?>
