<?PHP
###########################################################################
# 
# File:            taloLoggerGraph_functions.inc (PHP)
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.2c
#
# Date:            14.02.2015
#
# Functions:       getVersion
#                  generateLoggerGraphs
#                  readConfigurationFile
#                   
# Description:     House automation data logger graphing application for PHP. 
# 
#                  Implementing functions for the package.
#
# Requirements:    PHP 5.1 or newer (www.php.net)
#
#                  Suitable PDO database support for PHP.
#
#                  JpGraph PHP-graphing tool (http://jpgraph.net/)
#
# Version history: ** 20.10.2010 v0.1a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 21.10.2010 v0.1b (Olli Lammi) **
#                  Small bug fixes. Database optimisations.
#
#                  ** 23.10.2010 v0.1c (Olli Lammi) **
#                  SQLite version.
#
#                  ** 03.11.2010 v0.1d (Olli Lammi) **
#                  Changes to legend.
#
#                  ** 04.11.2010 v1.0a (Olli Lammi) **
#                  First version released to the public.
#
#                  ** 05.11.2010 v1.0b (Olli Lammi) **
#                  Bug fixes and support for non PDO 
#                  database connection for SQLite.
#
#                  ** 15.11.2010 v1.0c (Olli Lammi) **
#                  Support for IMAGE type graphs.
#
#                  ** 15.11.2010 v1.0d (Olli Lammi) **
#                  Added feature to hide the calendar.
#
#                  ** 16.11.2010 v1.0e (Olli Lammi) **
#                  Chart image size in parameter.
#
#                  ** 27.05.2011 v1.0f (Olli Lammi) **
#                  Counter type parameter added.
#
#                  ** 07.11.2011 v1.0g (Olli Lammi) **
#                  Added logic for handling counter
#                  value overflow. Added cumulative type
#                  counter series.
#
#                  ** 05.12.2011 v1.0h (Olli Lammi) **
#                  Moving state value bars to background.
#
#                  ** 15.01.2013 v1.1a (Olli Lammi) **
#                  Support for taloLogger native DB structure.
#
#                  ** 05.02.2013 v1.1b (Olli Lammi) **
#                  Running 24 h log. Added counter max value.
#
#                  ** 15.05.2014 v1.1c (Olli Lammi) **
#                  Possibility to configure more than 2 Y-axises. 
#                  Automatic scaling for axises.
#
#                  ** 01.12.2014 v1.1c (Olli Lammi) **
#                  Image caching features. 
#
#                  ** 08.12.2014 v1.1d (Olli Lammi) **
#                  Fixed leaking array references from code. 
#
#                  ** 31.12.2014 v1.1e (Olli Lammi) **
#                  Fixed caching problem if current date is selected
#                  from calendar.
#
#                  ** 02.01.2015 v1.2a (Olli Lammi) **
#                  Browser side graphing.
#
#                  ** 09.01.2015 v1.2b (Olli Lammi) **
#                  Released browser side graphing.
#
###########################################################################

include('taloLoggerGraph_tempfile.inc');
include('taloLoggerGraph_database.inc');
include('taloLoggerGraph_htmlchart.inc');

function getVersion() {
    return 'v1.2c';
}

function xLabelFormat($aLabel) { 
    return strftime('%H:%M', $aLabel);
} 

function output_error($msg) {
    echo($msg . "\n");
}


function getColor($lowcolor, $highcolor, $lowvalue, $highvalue, $value) {
  if ($value === null) return array(127,127,127);
  if ($value <= $lowvalue) return $lowcolor;
  if ($value >= $highvalue) return $highcolor;
  $m = ($value - $lowvalue) / ($highvalue - $lowvalue);
  return array(0 => $lowcolor[0] + $m * ($highcolor[0] - $lowcolor[0]), 
               1 => $lowcolor[1] + $m * ($highcolor[1] - $lowcolor[1]),
               2 => $lowcolor[2] + $m * ($highcolor[2] - $lowcolor[2]));
}


function yearEhto($type, $timecol, $year) {
    if ($type == 'MYSQL') {
        return "(" . $timecol . " BETWEEN '" .
               $year . "-01-01 00:00:00' AND " .
               "'" . $year . "-12-31 23:59:59')";
    }       
    else if ($type == 'SQLITE') {
        return "(" . $timecol . " BETWEEN DATETIME('" .
               $year . "-01-01 00:00:00', 'utc') AND " .
               "DATETIME('" . $year . "-12-31 23:59:59', 'utc'))";
    }
}

                   
function monthEhto($type, $timecol, $year, $month) {
    $d = sprintf('%04d-%02d-', $year, $month);
    $month2 = $month + 1;
    $year2 = $year;
    if ($month2 > 12) {
        $month2 = 1;
        $year2 = $year2 + 1;
    }              
    $d2 = sprintf('%04d-%02d-', $year2, $month2);

    if ($type == 'MYSQL') {    
        return  "(" . $timecol . " BETWEEN '" . $d .
                "01 00:00:00' AND DATE_ADD('" . $d2 . "01 00:00:00', " .
                "INTERVAL - 1 SECOND))";
    }
    else if ($type == 'SQLITE') {
        return  "(" . $timecol . " BETWEEN DATETIME('" . $d .
                "01 00:00:00', 'utc') AND DATETIME('" . $d2 . "01 00:00:00', " .
                "'-1 second', 'utc'))";
    }
}


function dayEhto($type, $timecol, $year, $month, $day) {
    $d = sprintf('%04d-%02d-%02d', $year, $month, $day);

    if ($type == 'MYSQL') {
        return  "(" . $timecol . " BETWEEN '" . $d .
                " 00:00:00' AND '" . $d . " 23:59:59')";
    }
    else if ($type == 'SQLITE') {
        return  "(" . $timecol . " BETWEEN DATETIME('" . $d .
                " 00:00:00', 'utc') AND DATETIME('" . $d . " 23:59:59', 'utc'))";
    }
}

function uxtimeEhto($type, $timecol, $starttime, $endtime) {
    if ($type == 'MYSQL') {
        return  "(" . $timecol . " BETWEEN " . 
        		"FROM_UNIXTIME(" . sprintf('%d', $starttime) . ") AND " .
        		"FROM_UNIXTIME(" . sprintf('%d', $endtime) . "))"; 
    }
    else if ($type == 'SQLITE') {
        return  "(" . $timecol . " BETWEEN " .
                "DATETIME(" . sprintf('%d', $starttime) . ", 'unixepoch') AND " .
                "DATETIME(" . sprintf('%d', $endtime) . ", 'unixepoch'))";
    }
}

function nextFullHour($t) {
	$ltarr = localtime($t, true);
	$daystarttime = mktime(0, 0, 0, 
	                       $ltarr['tm_mon'] + 1, 
	                       $ltarr['tm_mday'],
	                       $ltarr['tm_year']+1900);
	$temp = ($t - $daystarttime) % 3600;
	if ($temp == 0) $temp = 3600;
	return $t + 3600 - $temp;
}

function getTimeStr($t) {
    return strftime('%a %d.%m.%Y %H:%M', $t);
}

function getAxisTitle($chart, $axis) {
	$axistitle = sprintf('AXIS_%d', $axis);
	if (!in_array($axis, $chart['has_axis'])) return '';
    if ($axis > 1) return $chart[$axistitle]['title'];
    return $chart['AXIS_1']['title'];
}


function getLegendText($values, $istoday, $lastvalue, $avg, $unit) {			
    $temps = '';			
	if ($values > 0) {
	    if ( ($values == 1 || $values == 2) && $istoday) {
	        $temps .= $lastvalue;
	    }

        if ($values == 2 && $istoday) {
            $temps .= '/';
        }

        if ( ($values == 1 && !$istoday) || $values == 2 || $values == 3) {
            $temps .= $avg;
        }
	
        if ($unit) {
        	$temps .= ' ' . $unit;
        }      
    }
	
	return $temps;
}

function handleCounterRotate($value, $newvalue) {
	$COUNTER_MAXES = array(100, 1000, 10000, 100000, 1000000, 10000000, 256, 65536, 4294967296);
	
	$mindiff = 0;
	$mincmax = 0;
	foreach($COUNTER_MAXES as $cmax) {
		if ($value < $cmax) {
			if ($mincmax == 0 || $cmax - $value < $mindiff) {
				$mindiff = $cmax - $value;
				$mincmax = $cmax;
			}
		}
	}

	# if rollback would have been less than 10% of the counter value area, 
	# consider that the counter has rolled back
	if (abs($value - $newvalue) < 0.1 * $mincmax) return $value;
	
    return $value - $mincmax;	
}


function writeImageValue($gdih, $xpos, $ypos, $color, $str) {
    $col = imagecolorallocate($gdih, $color[0], $color[1], $color[2]);
    imagestring($gdih, 2, $xpos, $ypos, $str, $col);
}

function getDbValue($arr, $key) {
	if (array_key_exists($key, $arr)) {
		return $arr[$key]; 
	}
	return null;
}

function getCacheId($config, $index, $starttime, $type = 'png') {
	$cache_timestr = strftime('%Y/%m/%d', $starttime);
	return $config['FILENAME'] . '/day/' . $cache_timestr . '/tlimage_' . $index . '.' . $type;
}

function checkForCachedImage($config, $cache_id) {
	if (file_exists($config['GENERAL']['CACHE_DIRECTORY'] . '/' . $cache_id)) {
		return true;
	}
	
	return false;
}

function insertVersionTextToImage($gdih) {
	$col = imagecolorallocate($gdih, 220, 220, 220);
	imagestring($gdih, 2, imagesx($gdih) - 130, imagesy($gdih) - 15,
				'taloLoggerGraph ' . getVersion(), $col);
}

function addImageItem(&$items, $type, $x, $y, $col, $data) {
	$tempitem = array();
	$tempitem['type'] = $type;
	$tempitem['x'] = $x;
	$tempitem['y'] = $y;
	$tempitem['color'] = $col;
	$tempitem['data'] = $data;
	
	$items[] = $tempitem;	
}

function generateLoggerGraphs($dbtype, $dbconn, $charts, $istoday, $isfullday, $starttime, $endtime, $calctime, $regenerateCache, $js_charts, $isCmdline) {
	# check for cached images, if all charts are available, load the IDs and return
	$is_cache_enabled = false;
	if (!$istoday && $isfullday && $charts['GENERAL']['CACHE_IMAGES']) $is_cache_enabled = true;
	if ($is_cache_enabled && !$regenerateCache) {
		$has_all_parts = true;
		$part_ids = array();

		$ind = 0;
		foreach($charts['CHART'] as &$chart) {			
			$ind++;
			
			$type = 'png';
			if ($js_charts) {
				$type = 'html';
			}
			$cache_id = getCacheId($charts, $ind, $starttime, $type);
			if (checkForCachedImage($charts, $cache_id)) {				
				$temparr = array();
				$temparr['id'] = 'cache_' . $cache_id;
				$temparr['type'] = $type;
				$temparr['content'] = null;
				$temparr['files'] = null;
		
				if ($isCmdline && $type == 'html' && $chart['TYPE'] == 'IMAGE') {
					// needs to tell where the background image is (not in cache) to be able to
					// copy it to the command line package
					$extrafiles = array();
					$tempitem = array();
					$tempitem['url'] = '##IMAGE##URL##';
					$tempitem['type'] = pathinfo($chart['BACKGROUND'], PATHINFO_EXTENSION);
					$tempitem['fname'] = $chart['BACKGROUND'];
					$extrafiles[] = $tempitem;
					$temparr['files'] = $extrafiles;
				}
					
				$part_ids[] = $temparr; 
			}
			else {
				# cache miss, need to load data from database and generate missing images
				$has_all_parts = false;
				$part_ids = array();
				break;
			}			
		}
		unset($chart);
		
		if ($has_all_parts) {
			return $part_ids;
		}
	}
		
    $LOG_TABLE = $charts['GENERAL']['DB_TABLE'];
    $LOG_TIMECOL = $charts['GENERAL']['DB_TIMECOL'];
    $DB_SCHEMA = $charts['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'];
    $PIC_SIZE_X = 800;
    $PIC_SIZE_Y = 694;
    $LEGENDROWHEIGHT = 18;
    
    # initialize series values in charts
    $cols = array();
    foreach($charts['CHART'] as &$chart) {
        if ($chart['TYPE'] == 'CHART') {
	        $num_state_series = 0;
	        foreach($chart['SERIES'] as &$ser) {
	            if ( !in_array($ser['column'], $cols) ) {
	                $cols[] = $ser['column'];
	            }
	            $ser['XARRAY'] = array();
	            $ser['YARRAY'] = array();
	            $ser['hasdata'] = false;
	
	            if ($ser['type'] == 'STATE') {
	                $num_state_series += 1;
	            }
	
	            if ($ser['type'] == 'COUNTER') {
	                $ser['oldtime'] = 0;
	                $ser['oldvalue'] = 0;
	            }
	            $ser['lastvalue'] = 0;
	            $ser['avgvalue'] = 0;
	            $ser['avgsum'] = 0;
	            $ser['min'] = null;
	            $ser['max'] = null;	            
	        }
	        unset($ser);
	        $chart['num_state_series'] = $num_state_series;
	        
	        foreach($chart['SERIES'] as &$ser) {
	        	if ($ser['type'] == 'STATE') {
	        		$ser['axis'] = max($chart['has_axis']);
	        	}
	        }
	        unset($ser);	                      
        }
        else if ($chart['TYPE'] == 'IMAGE') {
        	if (array_key_exists('TITLE', $chart)) {
        		$chart['TITLE']['color'] = getArrayColor(getColorHexString($chart['TITLE']['color']));
        	}
        	
	        foreach($chart['POSITION'] as &$pos) {
	            if ( !in_array($pos['column'], $cols) ) {
	                $cols[] = $pos['column'];
	            }
	 	
	            if ($pos['type'] == 'COUNTER') {
	                $pos['oldtime'] = 0;
	                $pos['oldvalue'] = 0;
	            }
	            if ($pos['type'] == 'COUNTER' || $pos['type'] == 'NORMAL') {
	                $pos['lowcolor'] = getArrayColor(getColorHexString($pos['lowcolor']));
	                $pos['highcolor'] = getArrayColor(getColorHexString($pos['highcolor'])); 
	            }
	            else if ($pos['type'] == 'STATE') {
	            	$pos['color'] = getArrayColor(getColorHexString($pos['color']));
	            }
	            $pos['lastvalue'] = null;
	            $pos['avgvalue'] = 0;
	            $pos['avgsum'] = 0;
	            $pos['min'] = null;
	            $pos['max'] = null;
	        }
	        unset($pos);
	        
	        foreach($chart['LABEL'] as &$label) {
	        	$label['color'] = getArrayColor(getColorHexString($label['color']));
	        }
	        unset($label);
        }
    }
    unset($chart);
 
    $chart_date_utime = 0;
	$chart_date_maxutime = 0;
	$positions = array();
	$posids = array();
    
	if ($DB_SCHEMA == 1) {
		$sqlstmt = "SELECT id, position_name FROM talo_positions";
    	if (!$dbconn->query($sqlstmt)) {
    		output_error('ERROR: Error running database query. Cannot load position labels from DB.');
    		exit(0);
    	}

    	while($dbrow = $dbconn->fetchRow()) {
    		$positions[$dbrow['id']] = $dbrow['position_name'];
    		if ( in_array($dbrow['position_name'], $cols) ) {
    			$posids[] = $dbrow['id'];
    		}	
   		}
    	$dbconn->freeResult();
	}

	$dehto = uxtimeEhto($dbtype, $LOG_TIMECOL, $starttime, $endtime);	
	if ($dbtype == 'MYSQL') {
		if ($DB_SCHEMA == 1) {    				
	    	$sqlstmt = "SELECT " .
	                   "UNIX_TIMESTAMP(" . $LOG_TIMECOL . ") AS unixaika, " .
	                   "position_id, value FROM " . $LOG_TABLE .  
	                   " WHERE " . $dehto . 
	    	           " AND position_id IN (" . join(",", $posids) . ")";
		}
		else {
	    	$sqlstmt = "SELECT " .
	                   "UNIX_TIMESTAMP(" . $LOG_TIMECOL . ") as unixaika, " .
	                   join(", ", $cols) . " FROM " . $LOG_TABLE . 
	                   " WHERE " . $dehto .
	                   " ORDER BY " . $LOG_TIMECOL;
		}
    }
    else if ($dbtype == 'SQLITE') {
    	if ($DB_SCHEMA == 1) {
	    	$sqlstmt = "SELECT " .
	                   "STRFTIME('%s', " . $LOG_TIMECOL . ") as unixaika, " .
	                   "position_id, value FROM " . $LOG_TABLE .  
	                   " WHERE " . $dehto .
	    	           " AND position_id IN (" . join(",", $posids) . ")";
		}
		else {
	        $sqlstmt = "SELECT " .
	                   "STRFTIME('%s', " . $LOG_TIMECOL . ") as unixaika, " .
	                   join(", ", $cols) . " FROM " . $LOG_TABLE . 
	                   " WHERE " . $dehto .
	                   " ORDER BY " . $LOG_TIMECOL;
		}
    }
    else {
        output_error('ERROR: Invalid database type.');
        exit(0);
    }
    
    if (!$dbconn->query($sqlstmt)) {
    	output_error('ERROR: Error running database query. Check database table and column names in configuration.');
    	exit(0);
    }

    $rows = array();
    
	if ($DB_SCHEMA == 1) {
		while ($dbrow = $dbconn->fetchRow()) {
			if (!array_key_exists($dbrow['unixaika'], $rows)) {
				$rows[$dbrow['unixaika']] = array();
				$rows[$dbrow['unixaika']]['unixaika'] = $dbrow['unixaika'];				
			}
			$rows[$dbrow['unixaika']][$positions[$dbrow['position_id']]] = $dbrow['value'];
		}
    	$dbconn->freeResult();
    	ksort($rows);
	}
	else {
		while ($dbrow = $dbconn->fetchRow()) {
			$rows[] = $dbrow;
		}
    	$dbconn->freeResult();
	}
    
    foreach($rows as $row) {
        if ($chart_date_utime == 0) {
            $chart_date_utime = $row['unixaika'];    
        }
    	if ($chart_date_maxutime < $row['unixaika']) {
            $chart_date_maxutime = $row['unixaika'];    
        }
        
        foreach($charts['CHART'] as &$chart) {
        	if ($chart['TYPE'] == 'CHART') {
	            $state = $chart['num_state_series'];
	            foreach($chart['SERIES'] as &$ser) {
	                $val = getDbValue($row, $ser['column']);
                    if ($val == null) {
                        if ($ser['type'] == 'STATE') $state -= 1; 
                        continue;
                    }
	
	                if ($ser['type'] == 'NORMAL') {
	                    $ser['XARRAY'][] = $row['unixaika'];
	                    $val *= $ser['factor'];
	                    $ser['YARRAY'][] = $val;
	                    $ser['lastvalue'] = $val;
	                    $ser['avgvalue'] += $val;
	                    $ser['avgsum'] += 1;
						if ($ser['min'] === null || $ser['min'] > $val) $ser['min'] = $val;
	                    if ($ser['max'] === null || $ser['max'] < $val) $ser['max'] = $val;	               
	                }
	                else if ($ser['type'] == 'STATE') {
	                    $ser['XARRAY'][] = $row['unixaika'];
	
	                    $estr = str_replace('%VALUE%', '$val', $ser['statement']);
	                    $estr = '$val = ' . $estr . ';';
	                    eval($estr); 
	                    if ( $val ) {
	                        $val = 1; 
	                        $ser['lastvalue'] = 'ON';
	                        $ser['avgvalue'] += 1;
	                    } 
	                    else {
	                        $val = 0; 
	                        $ser['lastvalue'] = 'OFF';
	                    }
	                    $ser['avgsum'] += 1;
	                    $ser['YARRAY'][] = $val * $state;
	                    
	                    $state -= 1;
	                }
	                else if ($ser['type'] == 'COUNTER') {
	                    $nowtime = $row['unixaika'];
	                    if ($val) {
	                        if ($ser['oldtime'] > 0) {
	                        	if ( $ser['cnttype'] == 0 || 
	                        	     ($ser['cnttype'] == 1 && $val != $ser['oldvalue']) || 
	                        	     $ser['cnttype'] == 2 ) {
		                            $ser['XARRAY'][] = ($nowtime + $ser['oldtime']) / 2;
		                            if ($ser['maxval'] > 0) {
										$tmpval = $val - $ser['oldvalue'];
										if (abs($tmpval) > ($ser['maxval'] / 2.0)) {
											if ($tmpval < 0) {
												$tmpval += $ser['maxval'];
											}
											else {
												$tmpval -= $ser['maxval'];
											}
										}
		                            }
		                            else {
		                        	    $tmpval = $val - $ser['oldvalue'];
		                        	    if ($val < $ser['oldvalue']) {
		                        	    	$tmpval = $val - handleCounterRotate($ser['oldvalue'], $val);
		                        	    }
		                            }
		                            $cval = $ser['factor'] * ($tmpval) / ($nowtime - $ser['oldtime']);
	            	                $ser['avgvalue'] += $cval * ($nowtime - $ser['oldtime']);
	                	            $ser['avgsum'] += $nowtime - $ser['oldtime'];
	                	            $ser['oldtime'] = $nowtime;
	                        		$ser['oldvalue'] = $val;
	                        		if ($ser['cnttype'] == 2) {
	                        			$ser['YARRAY'][] = $ser['avgvalue'];
	                        			$ser['lastvalue'] = $ser['avgvalue'];
	                        			if ($ser['min'] === null || $ser['min'] > $ser['avgvalue']) $ser['min'] = $ser['avgvalue'];
	                            	    if ($ser['max'] === null || $ser['max'] < $ser['avgvalue']) $ser['max'] = $ser['avgvalue'];	                        		
	                        		}
	                        		else {
										$ser['YARRAY'][] = $cval;
										$ser['lastvalue'] = $cval;
									    if ($ser['min'] === null || $ser['min'] > $cval) $ser['min'] = $cval;
	                            	    if ($ser['max'] === null || $ser['max'] < $cval) $ser['max'] = $cval;	                	            									
	                        		}
	                        	}
	                        }
	                        else {
	                        	$ser['oldtime'] = $nowtime;
	                        	$ser['oldvalue'] = $val;
	                        }	                        
	                    }
	                }
	            }
	            unset($ser);
            }
            else if ($chart['TYPE'] == 'IMAGE') {
	            foreach($chart['POSITION'] as &$pos) {
	            	$val = getDbValue($row, $pos['column']);
					if ($val == null) continue;
	
	                if ($pos['type'] == 'NORMAL') {
	                    $val *= $pos['factor'];
	                    $pos['lastvalue'] = $val;
	                    $pos['avgvalue'] += $val;
	                    $pos['avgsum'] += 1;
	                    if ($pos['min'] === null || $pos['min'] > $val) $pos['min'] = $val;
	                    if ($pos['max'] === null || $pos['max'] < $val) $pos['max'] = $val;
	                }
	                else if ($pos['type'] == 'STATE') {
	                    $estr = str_replace('%VALUE%', '$val', $pos['statement']);
	                    $estr = '$val = ' . $estr . ';';
	                    eval($estr); 
	                    if ( $val ) {
	                        $val = 1; 
	                        $pos['lastvalue'] = 'ON';
	                        $pos['avgvalue'] += 1;
	                    } 
	                    else {
	                        $val = 0; 
	                        $pos['lastvalue'] = 0;
	                    }
	                    $pos['avgsum'] += 1;
	                }
	                else if ($pos['type'] == 'COUNTER') {
	                    $nowtime = $row['unixaika'];
	                    if ($val) {
	                        if ($pos['oldtime'] > 0) {
	                        	if ( $pos['cnttype'] == 0 || 
	                        	     ($pos['cnttype'] == 1 && $val != $pos['oldvalue']) ||
	                        	     $pos['cnttype'] == 2) {                        	  	                        	     	
	                        	     if ($pos['maxval'] > 0) {
										$tmpval = $val - $pos['oldvalue'];
										if (abs($tmpval) > ($pos['maxval'] / 2.0)) {
											if ($tmpval < 0) {
												$tmpval += $pos['maxval'];
											}
											else {
												$tmpval -= $pos['maxval'];
											}
										}
		                            }
		                            else {
		                        	    $tmpval = $val - $pos['oldvalue'];
		                        	    if ($val < $pos['oldvalue']) {
		                        	    	$tmpval = $val - handleCounterRotate($pos['oldvalue'], $val);
		                        	    }
		                            }
	                            	$cval = $pos['factor'] * ($tmpval) / ($nowtime - $pos['oldtime']);
	                            	$pos['avgvalue'] += $cval * ($nowtime - $pos['oldtime']);
	                            	$pos['avgsum'] += $nowtime - $pos['oldtime'];
	                            	if ($pos['min'] === null || $pos['min'] > $cval) $pos['min'] = $cval;
	                            	if ($pos['max'] === null || $pos['max'] < $cval) $pos['max'] = $cval;
	                            	$pos['oldtime'] = $nowtime;
	                        		$pos['oldvalue'] = $val;
	                        		if ($pos['cnttype'] == 2) {
	                        			$pos['lastvalue'] = $pos['avgvalue'];	
	                        		}
	                        		else {
	                        			$pos['lastvalue'] = $cval;	                        			
	                        		}	                            	
	                        	}
	                        }
	                        else {
	                        	$pos['oldtime'] = $nowtime;
	                        	$pos['oldvalue'] = $val;
	                        }
	                    }
	                }
	            }
	            unset($pos);
            }
        }
        unset($chart);
    }

    # averages
    foreach($charts['CHART'] as &$chart) {
        if ($chart['TYPE'] == 'CHART') {
	        foreach($chart['SERIES'] as &$ser) {
	            if ($ser['type'] == 'NORMAL' || $ser['type'] == 'COUNTER') {
	            	if ($ser['lastvalue'] === null) {
	            		$ser['lastvalue'] = '---';
	            	}
	            	else {
		                $ser['lastvalue'] = sprintf('%.2f', $ser['lastvalue']);
	            	}
	                if ($ser['avgsum']) {
	                	if ($ser['type'] == 'COUNTER' && $ser['cnttype'] == 2) {
	                		$ser['avg'] = sprintf('%.2f', $ser['avgvalue'] / $ser['avgsum'] * 
	                		                              ($endtime - $starttime));
	                	}
	                	else {
	                	    $ser['avg'] = sprintf('%.2f', $ser['avgvalue'] / $ser['avgsum']);
	                	}
	                }
	                else {
	                	$ser['avg'] = '---';
	                }
	            }
	            else if ($ser['type'] == 'STATE') {
	            	if ($ser['avgsum']) {
	                	$ser['avg'] = sprintf('%.2f %%', $ser['avgvalue'] / $ser['avgsum'] * 100.0);
                	}
                	else {
                		$ser['avg'] = '---';
                	}
	            }
	        }
	        unset($ser);
        }
        else if ($chart['TYPE'] == 'IMAGE') {
	        foreach($chart['POSITION'] as &$pos) {
	            if ($pos['type'] == 'NORMAL' || $pos['type'] == 'COUNTER') {
	            	if ($pos['lastvalue'] !== null) {
		                $pos['lastvalue'] = sprintf('%.2f', $pos['lastvalue']);
	            	}
	            	if ($pos['min'] !== null) {
		                $pos['min'] = sprintf('%.2f', $pos['min']);
	            	}
	            	if ($pos['max'] !== null) {
		                $pos['max'] = sprintf('%.2f', $pos['max']);
	            	}
	            	if ($pos['avgsum']) {
	                	if ($pos['type'] == 'COUNTER' && $pos['cnttype'] == 2) {
	                		$pos['avg'] = sprintf('%.2f', $pos['avgvalue'] / $pos['avgsum'] * 
	                		                              ($endtime - $starttime));
	                	}
	                	else {
	                	    $pos['avg'] = sprintf('%.2f', $pos['avgvalue'] / $pos['avgsum']);
	                	}
	                }
                	else {
                		$pos['avg'] = null;
                	}
	            }
	            else if ($pos['type'] == 'STATE') {
	            	if ($pos['avgsum']) {
	                	$pos['avg'] = sprintf('%.2f', $pos['avgvalue'] / $pos['avgsum'] * 100.0);
                	}
                	else {
                		$pos['avg'] = null;
                	}
	            }
	        }
	        unset($pos);
        }
    }
    unset($chart);

    # axis scales
    foreach($charts['CHART'] as &$chart) {
        if ($chart['TYPE'] == 'CHART') {
        	$axis_min = array();
    		$axis_max = array();
        	
	        foreach($chart['SERIES'] as &$ser) {
	            if ($ser['type'] == 'NORMAL' || $ser['type'] == 'COUNTER') {
	            	$axis = $ser['axis'];
	            	if (!array_key_exists($axis, $axis_min)) $axis_min[$axis] = null;
	            	if (!array_key_exists($axis, $axis_max)) $axis_max[$axis] = null; 
	            	if ($axis_min[$axis] === null || $axis_min[$axis] > $ser['min']) $axis_min[$axis] = $ser['min'];
	            	if ($axis_max[$axis] === null || $axis_max[$axis] < $ser['max']) $axis_max[$axis] = $ser['max'];
	            }
	        }
	        unset($ser);
	        
	        foreach($chart['has_axis'] as &$axis) {
	        	$tempmin = $axis_min[$axis];
	            $tempmax = $axis_max[$axis];
	            $axistitle = sprintf('AXIS_%d', $axis);
	            $margin = ($tempmax - $tempmin) / 8.0;	            
	        	if ($chart[$axistitle]['min'] == 'auto') {
	        		$chart[$axistitle]['min'] = $tempmin - $margin;
	        	}
	        	if ($chart[$axistitle]['max'] == 'auto') {
	        		$chart[$axistitle]['max'] = $tempmax + $margin;
	        	}
	        	if ($chart[$axistitle]['min'] > $chart[$axistitle]['max']) {
	        		$temp = $chart[$axistitle]['max'];
	        		$chart[$axistitle]['max'] = $chart[$axistitle]['min'];
	        		$chart[$axistitle]['min'] = $temp;
	        	}
	        		        	
	        	if ($chart[$axistitle]['matick'] == 'auto') {
	        		$tickscale = ($chart[$axistitle]['max'] - $chart[$axistitle]['min']) / 6.0;	        		
	        		$tickscale = pow(10, round(log10($tickscale)));	        		
	        		$chart[$axistitle]['matick'] = $tickscale;
	        	}	        	
	        	if ($chart[$axistitle]['mitick'] == 'auto') {
	        		$chart[$axistitle]['mitick'] = $chart[$axistitle]['matick'] / 10.0;
	        	}
	        }
	        unset($axis);
	        
	        foreach($chart['SERIES'] as &$ser) {
	            if ($ser['type'] == 'STATE') {
					$axistitle = sprintf('AXIS_%d', $ser['axis']);
						            	            	
                    $STATEVALUE_STEPPING = ($chart[$axistitle]['max'] - $chart[$axistitle]['min']) / 60.0;
                    foreach($ser['YARRAY'] as &$val) {
                    	$val = $val * $STATEVALUE_STEPPING + $chart[$axistitle]['min'];
                    }
                    unset($val);           	
	            }
	        }
	        unset($ser);	        
        }
    }
    unset($chart);
        
    $idarr = array();
    $ind = 0;
    foreach($charts['CHART'] as &$chart) {
    	$ind++;
    	$gdih = null;
    	$htmlresult = null;
    	$extrafiles = null;
    	$should_be_cached_with_id = null;
    	$chart_result_type = 'png';
    	if ($js_charts) {
    		$chart_result_type = 'html';
    	}
    	 
    	#check if the image is in the cache
    	if ($is_cache_enabled) {    	     		
    		$cache_id = getCacheId($charts, $ind, $starttime, $chart_result_type);
            if (!$regenerateCache && checkForCachedImage($charts, $cache_id)) {
            	$temparr = array();
	    		$temparr['id'] = 'cache_' . $cache_id;
	    		$temparr['type'] = $chart_result_type;
	    		$temparr['content'] = null;
	    		$temparr['files'] = null;

	    		if ($isCmdline && $chart_result_type == 'html' && $chart['TYPE'] == 'IMAGE') {
					// needs to tell where the background image is (not in cache) to be able to 
					// copy it to the command line package
	    			$extrafiles = array();
	    			$tempitem = array();
	    			$tempitem['url'] = '##IMAGE##URL##';
	    			$tempitem['type'] = pathinfo($chart['BACKGROUND'], PATHINFO_EXTENSION);
	    			$tempitem['fname'] = $chart['BACKGROUND'];
	    			$extrafiles[] = $tempitem;
	    			$temparr['files'] = $extrafiles;
	    		}
	    			   
	    		$idarr[] = $temparr;
	    		continue;
	    	}
	    	else {
	    		$should_be_cached_with_id = $cache_id;
	    		$cache_fname = $charts['GENERAL']['CACHE_DIRECTORY'] . '/' . $should_be_cached_with_id;
	    		if (!is_dir(dirname($cache_fname))) {
	    			mkdir(dirname($cache_fname), 0775, true);
	    		}	    		 
	    	}
    	}
    	
        if ($chart['TYPE'] == 'CHART') {
        	$axis_has_data = array();
    
        	foreach($chart['SERIES'] as &$ser) {
        		if (count($ser['YARRAY']) > 0) {
        			$ser['hasdata'] = true;
        			if (!in_array($ser['axis'], $axis_has_data)) $axis_has_data[] = $ser['axis'];
        		}
        	}
        	unset($ser);
        	sort($axis_has_data);
        	$axis_map = array();
        	$i = 1;
        	foreach($axis_has_data as $axis) {
        		$axis_map[$axis] = $i;
        		$i += 1;
        	}
        	        	    	        	
        	$xsize = $PIC_SIZE_X;
        	$ysize = $PIC_SIZE_Y;
			if (array_key_exists('SIZE', $chart)) {
				$xsize = $chart['SIZE']['xsize'];
				$ysize = $chart['SIZE']['ysize'];
			}
        	
        	$legendrows = 0;
	        if ($chart['LEGEND']['cols'] > 0) {
	            $legendrows = ceil(count($chart['SERIES']) / $chart['LEGEND']['cols']);
	        }

	        $title = $chart['TITLE'] . ' ';
	        if ($isfullday) {
	        	$title .= strftime('%a %d.%m.%Y', $chart_date_utime);
	        }
	        else {
	        	$sday = strftime('%a %d.%m.%Y', $starttime);
	        	$eday = strftime('%a %d.%m.%Y', $endtime);
	        	if ($sday != $eday) {
	        		$title .= $sday . ' - ' . $eday;
	        	}
	        	else {
	        		$title .= $eday;
	        	}
	        }

	        foreach($chart['SERIES'] as &$ser) {
	        	if ($ser['hasdata']) {	        	        			
        			$tempunit = '';
        			if ($ser['type'] == 'NORMAL' || $ser['type'] == 'COUNTER') {
        				$tempunit = getAxisTitle($chart, $ser['axis']);
        			}
        			
        			$temps = getLegendText($chart['LEGEND']['values'], $istoday, $ser['lastvalue'], $ser['avg'], $tempunit);
        			if (strlen($temps) > 0) $temps = ': ' . $temps;

        			$ser['legend_text'] = $ser['legend'] . $temps;
        			if (array_key_exists('axis', $ser) && $axis_map[$ser['axis']] > 1 && $ser['type'] != 'STATE') {
        				$ser['legend_text'] = $ser['legend'] . ' (' . $axis_map[$ser['axis']] . ')' . $temps;
        			}	        	
	        	}
	        }
	        unset($ser);
	        
	        if ($chart_result_type == 'png') {
	        	$firstaxistitle = 'AXIS_1';
	        	if (count($axis_has_data) > 0) {
	        		$firstaxistitle = sprintf('AXIS_%d', $axis_has_data[0]);
	        	}

	        	$axismargin = 40;
	        	if (count($axis_has_data) > 2) {
	        		$axismargin += (count($axis_has_data) - 2) * 50;
	        	}

	        	include_once ('jpgraph.php');
	        	include_once ('jpgraph_line.php');
	        	
	        	$graph = new Graph($xsize, $ysize + $legendrows * $LEGENDROWHEIGHT, 'auto');
		        $graph->SetScale('linlin', $chart[$firstaxistitle]['min'], $chart[$firstaxistitle]['max'], $starttime, $endtime);
	
		        $graph->img->SetMargin(40,$axismargin,20,74 + $legendrows * $LEGENDROWHEIGHT);
		        $graph->SetMarginColor('white');
		        $graph->xaxis->SetPos('min');
		        $graph->xaxis->SetTitlemargin(36 + $legendrows * $LEGENDROWHEIGHT);
		        
		        $graph->xaxis->SetTitle($title, 'middle');
			    $graph->xaxis->scale->ticks->Set(60*60,15*60);
			    	        
			   	foreach($axis_has_data as $axis) {
			   		$axisind = $axis_map[$axis];
			   		$axistitle = sprintf('AXIS_%d', $axis);
			   		$axisunit = $chart[$axistitle]['title'];
			   		if (count($axis_has_data) > 2 && $axisind > 1) {
			   			$axisunit .= ' (' . $axisind . ')';
			   		}
			   		 
			   		if ($axisind == 1) {
		        		$graph->yaxis->title->Set($chart[$axistitle]['title']);
		        		$graph->yaxis->scale->ticks->Set($chart[$axistitle]['matick'],$chart[$axistitle]['mitick']);	        	
		        	}
		        	else {	        		 	        		
		        		$axnum = $axisind - 2;
		        		$graph->SetYScale($axnum, 'lin', $chart[$axistitle]['min'], $chart[$axistitle]['max']);
			        	$graph->ynaxis[$axnum]->title->Set($axisunit);
			        	$graph->ynaxis[$axnum]->scale->ticks->Set($chart[$axistitle]['matick'],$chart[$axistitle]['mitick']);
			        	$graph->ynaxis[$axnum]->SetTitleSide(SIDE_RIGHT);
			        	$graph->ynaxis[$axnum]->SetTitlemargin(23); 
		        	}
		        }
			    	
			    $graph->xgrid->Show(true, true);
			    $graph->ygrid->Show(true, true);
			    $graph->ygrid->SetColor('#E0E0E0', '#F0F0F0');
			    $graph->xgrid->SetColor('#E0E0E0', '#F0F0F0');
			    $graph->xaxis->scale->ticks->SetColor('black','black'); 
			    $graph->xaxis->scale->ticks->SupressMinorTickMarks(false);
			
			    $graph->xaxis->SetLabelFormatCallback('xLabelFormat');
			    $graph->xaxis->SetLabelAngle(90);
			
			    $graph->legend->SetColumns($chart['LEGEND']['cols']);
			    $graph->legend->Pos(0.5, ($ysize - 33.0) / ($ysize + $legendrows * $LEGENDROWHEIGHT), 'center', 'top');
		
			    foreach($chart['SERIES'] as &$ser) {
			        if ($ser['hasdata']) {
				        $lp = new LinePlot($ser['YARRAY'], $ser['XARRAY']);
				        $lp->SetColor($ser['color']);
				        $lp->SetWeight(1); 
				
				        if ($ser['type'] == 'STATE') {
				            $lp->SetFillColor($ser['color']);
				            $lp->SetFillFromYMin(TRUE);
				            $lp->SetStepStyle();
				        }

				        if ($legendrows > 0) {
				        	$lp->SetLegend($ser['legend_text']);
				        }
				        
				        if (!array_key_exists('axis', $ser)) {
				        	$graph->Add($lp);
				        }
				        else {
							$axisind = $axis_map[$ser['axis']];
	
					        if ($axisind > 1) {
					            $axnum = $axisind - 2;
					            $graph->AddY($axnum, $lp); 
					        }
					        else { 
					            $graph->Add($lp);
					        }
				        }
			        }
			    }
			    unset($ser);
	
	   	        $gdih = $graph->Stroke(_IMG_HANDLER);
	   	        
	   	        insertVersionTextToImage($gdih);
	        }
	        else if ($chart_result_type == 'html') {

	        	foreach($chart['SERIES'] as &$ser) {
		        	$ser['color'] = getColorHexString($ser['color']);
	        	}
	        	unset($ser);
	        	
				$htmlresult = getHtmlChartDiv($chart, $xsize, $ysize, $legendrows, $starttime, $endtime, $title, $axis_has_data, $axis_map);
	        }   	        
	        
		}
		else if ($chart['TYPE'] == 'IMAGE') {
			$items = array();
			
			if (array_key_exists('TITLE', $chart)) {
				$temps = $chart['TITLE']['title'];
				if ($istoday) {
					$temps .= ' ' . strftime('%a %d.%m.%Y %H:%M', $chart_date_maxutime);
				}
				else {
					if ($isfullday) {
						$temps .= ' ' . strftime('%a %d.%m.%Y', $chart_date_utime);
					}
					else {
						$temps .= ' ' . strftime('%a %d.%m.%Y %H:%M', $starttime) . ' - ' . strftime('%a %d.%m.%Y %H:%M', $endtime);
					}
				}
				
				addImageItem($items, 'text', $chart['TITLE']['xpos'], $chart['TITLE']['ypos'], $chart['TITLE']['color'], $temps);
			}

			foreach($chart['POSITION'] as &$pos) {
				if ($pos['type'] == 'STATE' && $pos['subtype'] == 'VAL') {
					if ($istoday && $pos['lastvalue']) {
						addImageItem($items, 'circle', $pos['xpos'], $pos['ypos'], $pos['color'], $pos['size']);
					}
				}
				else if ($pos['type'] == 'STATE' && $pos['subtype'] == 'AVG') {
					$col = array(127,127,127);
					$temps = '---';
					if ($pos['avg'] !== null) {
						$col = $pos['color'];
						$temps = $pos['avg'] . '%';
					}
				    addImageItem($items, 'text', $pos['xpos'], $pos['ypos'], $col, $temps);
				}
				else if ($pos['subtype'] == 'VAL' && in_array($pos['type'], array('NORMAL', 'COUNTER'))) {
					$col = array(127,127,127);
					$temps = '---';
					if ($istoday && $pos['lastvalue'] !== null) {
						$val = $pos['lastvalue'];
						$col = getColor($pos['lowcolor'], $pos['highcolor'],
								$pos['lowvalue'], $pos['highvalue'], $val);
						$temps = $val . $pos["unit"];
					}
				    addImageItem($items, 'text', $pos['xpos'], $pos['ypos'], $col, $temps);
				}
				else if (in_array($pos['subtype'], array('MIN', 'MAX', 'AVG')) && in_array($pos['type'], array('NORMAL', 'COUNTER'))) {
					$col = array(127,127,127);
					$temps = '---';
					$val = null;
					if ($pos['subtype'] == 'MIN') $val = $pos['min'];
					else if ($pos['subtype'] == 'MAX') $val = $pos['max'];
					else if ($pos['subtype'] == 'AVG') $val = $pos['avg'];

					if ($val !== null) {
						$col = getColor($pos['lowcolor'], $pos['highcolor'],
								$pos['lowvalue'], $pos['highvalue'], $val);
						$temps = $val . $pos["unit"];
					}
				    addImageItem($items, 'text', $pos['xpos'], $pos['ypos'], $col, $temps);
				}
			}
			unset($pos);
			
			foreach($chart['LABEL'] as &$label) {
				addImageItem($items, 'text', $label['xpos'], $label['ypos'], $label['color'], $label['label']);
			}
			unset($label);
							
			if ($chart_result_type == 'png') {
				$gdih = imagecreatefrompng($chart['BACKGROUND']);
				if ($gdih) {    
					foreach($items as &$item) {
						if ($item['type'] == 'text') {
							writeImageValue($gdih, $item['x'], $item['y'], $item['color'], $item['data']);
						}
						else if ($item['type'] == 'circle') {
							$col = imagecolorallocate($gdih, $item['color'][0], $item['color'][1], $item['color'][2]);
							imagefilledellipse($gdih, $item['x'], $item['y'], $item['data'], $item['data'], $col);
						}
					}
					unset($item);

					insertVersionTextToImage($gdih);
				}
				else {
					error_log("ERROR: Unable to load png file: " . $chart['BACKGROUND']);
				}
			}
			else if ($chart_result_type == 'html') {				
				$htmlresult = getHtmlImageDiv($chart, $items);
				
				$extrafiles = array();
				$tempitem = array();
				$tempitem['url'] = '##IMAGE##URL##';
				$tempitem['type'] = pathinfo($chart['BACKGROUND'], PATHINFO_EXTENSION);
				$tempitem['fname'] = $chart['BACKGROUND'];
				$extrafiles[] = $tempitem;
			}
		}

		$tempcontent = null;
	    $id = null;
	    if ($should_be_cached_with_id) {
	    	$cache_fname = $charts['GENERAL']['CACHE_DIRECTORY'] . '/' . $should_be_cached_with_id;
	    	if ($chart_result_type == 'png') {	    		 
		    	if (imagepng($gdih, $cache_fname)) {
		    		$id = 'cache_' . $should_be_cached_with_id;
		    	}
	    	}
	    	else if ($chart_result_type == 'html') {
	    		if (file_put_contents($cache_fname, $htmlresult)) {
	    			$id = 'cache_' . $should_be_cached_with_id;
	    		}
	    	}
	    }

	    if (!$id) {
	    	if ($chart_result_type == 'png') {	    	
			    $id = getTempId('png');		
			    imagepng($gdih, getImagePath($id, 'png'));
	    	}
	    	else if ($chart_result_type == 'html') {
				$id = '';
				$tempcontent = $htmlresult;
	    	}
	    }

	    if ($gdih) {
	    	imagedestroy($gdih);
	    	$gdih = null;
	    }

	    if ($htmlresult != null) {
	    	$htmlresult = null;
	    }
	    
	    $temparr = array();
	    $temparr['id'] = $id;
	    $temparr['type'] = $chart_result_type;
	    $temparr['content'] = $tempcontent;
	    $temparr['files'] = $extrafiles;
	    $idarr[] = $temparr;
	    $tempcontent = null;
    }
    unset($chart);

    return $idarr;
}

function findURLForFile($fname) {
	if (!isset($_SERVER) || !$_SERVER['DOCUMENT_ROOT']) return $fname;

	$len = strlen($_SERVER['DOCUMENT_ROOT']);
	if (substr($fname, 0, $len) === $_SERVER['DOCUMENT_ROOT']) {
		return substr($fname, $len);
	}

	return $fname;
}

function handleAXISConfRow($value) {
    # should be: min:max:title:majortick:minortick
    # example: 0.0:6.0:kW:0.5:0.1
    $temp = explode(':', $value, 5);
    if (count($temp) != 5) return null;
    if (!is_numeric($temp[0]) && $temp[0] != 'auto') return null;
    if (!is_numeric($temp[1]) && $temp[1] != 'auto') return null;
    if (!is_numeric($temp[2]) && $temp[2] != 'auto') return null;
    if (!is_numeric($temp[3]) && $temp[3] != 'auto') return null;

    $res = array();
    $res['min'] = trim($temp[0]);
    $res['max'] = trim($temp[1]);
    $res['matick'] = trim($temp[2]);
    $res['mitick'] = trim($temp[3]);
    $res['title'] = trim($temp[4]);
    return $res;
}


function handleSIZEConfRow($value) {
    # should be: xsize:ysize
    # example: 800:694
    $temp = explode(':', $value);
    if (count($temp) != 2) return null;
    if (!is_numeric($temp[0])) return null;
    if (!is_numeric($temp[1])) return null;
  
    $res = array();
    $res['xsize'] = trim($temp[0]);
    $res['ysize'] = trim($temp[1]);
    return $res;
}


function handleLEGENDConfRow($value) {
    # should be: cols:values
    # example: 6:3
    $temp = explode(':', $value, 2);
    if (count($temp) != 2) return null;
    if (!is_numeric($temp[0])) return null;
    if (!is_numeric($temp[1])) return null;

    $res = array();
    $res['cols'] = trim($temp[0]);
    $res['values'] = trim($temp[1]);

    if ($res['values'] != 0 && $res['values'] != 1 &&
        $res['values'] != 2 && $res['values'] != 3) {
        return null;
    }

    return $res;
}


function handleSERIESConfRow($value) {
    $temp = explode(':', $value);
    
    if (count($temp) < 1) return null;

    $type = trim($temp[0]);
    if ($type == 'NORMAL') {
        if (count($temp) != 6) return null;

        if (!is_numeric($temp[4])) return null;
        if (!is_numeric($temp[5])) return null;

        $res = array();
        $res['type'] = $type;
        $res['column'] = trim($temp[1]);
        $res['legend'] = trim($temp[2]);
        $res['color'] = trim($temp[3]);
        $res['axis'] = trim($temp[4]);
        $res['factor'] = trim($temp[5]);

        $temp = $res;
    }
    else if ($type == 'COUNTER') {
    	$cnt = count($temp);
        if ($cnt < 7 || $cnt > 8) return null;

        if (!is_numeric($temp[4])) return null;
        if (!is_numeric($temp[5])) return null;
		if (!is_numeric($temp[6]) || ($temp[6] != 0 && $temp[6] != 1 && $temp[6] != 2)) return null;
  		if ($cnt == 8 && !is_numeric($temp[7])) return null;
				
        $res = array();
        $res['type'] = $type;
        $res['column'] = trim($temp[1]);
        $res['legend'] = trim($temp[2]);
        $res['color'] = trim($temp[3]);
        $res['axis'] = trim($temp[4]);
        $res['factor'] = trim($temp[5]);
        $res['cnttype'] = trim($temp[6]);
        $res['maxval'] = '0';
        if ($cnt == 8) {
        	$res['maxval'] = trim($temp[7]) + 1;
        }
        
        $temp = $res;
    }
    else if ($type == 'STATE') {
        if (count($temp) != 5) return null;

        $res = array();
        $res['type'] = $type;
        $res['column'] = trim($temp[1]);
        $res['legend'] = trim($temp[2]);
        $res['color'] = trim($temp[3]);
        $res['statement'] = trim($temp[4]);

        $temp = $res;
    }
    else {
        return null;
    }

    return $temp;
}


function handleIMAGETITLEConfRow($value) {
    # should be: xpos:ypos:color:title
    # example: 200:100:black:Image title
    $temp = explode(':', $value, 4);
    if (count($temp) != 4) return null;
    if (!is_numeric($temp[0])) return null;
    if (!is_numeric($temp[1])) return null;

    $res = array();
    $res['xpos'] = trim($temp[0]);
    $res['ypos'] = trim($temp[1]);
    $res['color'] = trim($temp[2]);
    $res['title'] = trim($temp[3]);
    return $res;
}


function handlePOSITIONConfRow($value) {
    $temp = explode(':', $value);
        
    if (count($temp) < 1) return null;

    $type = trim($temp[0]);
    if ($type == 'NORMAL') {
        if (count($temp) != 11) return null;

        if (!is_numeric($temp[2])) return null;
        if (!is_numeric($temp[3])) return null;
		if (!is_numeric($temp[5])) return null;
		if (!is_numeric($temp[8])) return null;
		if (!is_numeric($temp[9])) return null;

        $res = array();
        $res['type'] = $type;
        $res['column'] = trim($temp[1]);
        $res['xpos'] = trim($temp[2]);
        $res['ypos'] = trim($temp[3]);
        $res['subtype'] = strtoupper(trim($temp[4]));
        $res['factor'] = trim($temp[5]);
        $res['lowcolor'] = trim($temp[6]);
		$res['highcolor'] = trim($temp[7]);
		$res['lowvalue'] = trim($temp[8]);
		$res['highvalue'] = trim($temp[9]);
        $res['unit'] = trim($temp[10]);

        if (!in_array($res['subtype'], array('VAL', 'AVG', 'MIN', 'MAX'))) return null;
        
        $temp = $res;
    }
    else if ($type == 'COUNTER') {
    	$cnt = count($temp);
        if ($cnt < 12 || $cnt > 13) return null;

        if (!is_numeric($temp[2])) return null;
        if (!is_numeric($temp[3])) return null;
		if (!is_numeric($temp[5])) return null;
		if (!is_numeric($temp[8])) return null;
		if (!is_numeric($temp[9])) return null;
		if (!is_numeric($temp[11]) || ($temp[11] != 0 && $temp[11] != 1 && $temp[11] != 2)) return null;
		if ($cnt == 13 && !is_numeric($temp[12])) return null;
		
        $res = array();
        $res['type'] = $type;
        $res['column'] = trim($temp[1]);
        $res['xpos'] = trim($temp[2]);
        $res['ypos'] = trim($temp[3]);
        $res['subtype'] = strtoupper(trim($temp[4]));
        $res['factor'] = trim($temp[5]);
        $res['lowcolor'] = trim($temp[6]);
		$res['highcolor'] = trim($temp[7]);
		$res['lowvalue'] = trim($temp[8]);
		$res['highvalue'] = trim($temp[9]);
        $res['unit'] = trim($temp[10]);
        $res['cnttype'] = trim($temp[11]);
        $res['maxval'] = '0';
        if ($cnt == 13) {
        	$res['maxval'] = trim($temp[12]) + 1;
        }
        	
        if (!in_array($res['subtype'], array('VAL', 'AVG', 'MIN', 'MAX'))) return null;
        
        $temp = $res;
    }
    else if ($type == 'STATE') {
        if (count($temp) != 8) return null;

		if (!is_numeric($temp[2])) return null;
		if (!is_numeric($temp[3])) return null;
		if (!is_numeric($temp[6])) return null;
		
        $res = array();
        $res['type'] = $type;
        $res['column'] = trim($temp[1]);
        $res['xpos'] = trim($temp[2]);
        $res['ypos'] = trim($temp[3]);
        $res['subtype'] = strtoupper(trim($temp[4]));
        $res['color'] = trim($temp[5]);
		$res['size'] = trim($temp[6]);
		$res['statement'] = trim($temp[7]);

        if (!in_array($res['subtype'], array('VAL', 'AVG'))) return null;
        
        $temp = $res;
    }
    else {
        return null;
    }

    return $temp;
}


function handleLABELConfRow($value) {
	# example:  xpos:ypos:color:label
	$temp = explode(':', $value, 4);
    if (count($temp) != 4) return null;
    if (!is_numeric($temp[0])) return null;
    if (!is_numeric($temp[1])) return null;

    $res = array();
    $res['xpos'] = trim($temp[0]);
    $res['ypos'] = trim($temp[1]);
    $res['color'] = trim($temp[2]);
    $res['label'] = trim($temp[3]);

    return $res;
}


function readConfigurationFile($fname) {
    $charts = array();
    $charts['GENERAL'] = array();
    $charts['CHART'] = array();

    $generalvalues = array();
    $generalvalues[] = 'DB_TYPE';
    $generalvalues[] = 'DB_TALOLOGGER_SCHEMA_VERSION';
    $generalvalues[] = 'DB_TABLE';    
    $generalvalues[] = 'DB_TIMECOL';    
    $generalvalues[] = 'MYSQLDB_HOST';    
    $generalvalues[] = 'MYSQLDB_USER';    
    $generalvalues[] = 'MYSQLDB_PASSWD';    
    $generalvalues[] = 'MYSQLDB_NAME';    
    $generalvalues[] = 'SQLITEDB_NAME';    
    $generalvalues[] = 'HIDE_CALENDAR'; 
    $generalvalues[] = 'CACHE_IMAGES'; 
    $generalvalues[] = 'CACHE_DIRECTORY'; 
    $generalvalues[] = 'CACHE_URL_PATH';
    $generalvalues[] = 'JAVASCRIPT_CHARTS';
    
    $generallists = array();

    $sectvalues = array();
    $sectvalues['CHART'] = array();
    $sectvalues['CHART'][] = 'TITLE';
    $sectvalues['CHART'][] = 'AXIS_1';
    $sectvalues['CHART'][] = 'LEGEND';
    $sectvalues['CHART'][] = 'SIZE';
    
    $sectlists = array();
    $sectlists['CHART'] = array();
    $sectlists['CHART'][] = 'SERIES';

    $sectvalues['IMAGE'] = array();
    $sectvalues['IMAGE'][] = 'BACKGROUND';
    $sectvalues['IMAGE'][] = 'TITLE';
    
    $sectlists['IMAGE'] = array();
    $sectlists['IMAGE'][] = 'POSITION';
	$sectlists['IMAGE'][] = 'LABEL';

    $sect = null;
    
    $lines = file($fname, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

    if (!$lines) {
        return array('stat' => -1, 'msg' => 'ERROR: Cannot load configuration file ' . $fname);
    }

    $charts['FILENAME'] = basename($fname);
    
    foreach($lines as $line) {
        if (!$line) continue;
        $line = trim($line);
        if (strlen($line) <= 0 || $line[0] == '#') continue;

        if ($line == '*CHART*' || $line == '*IMAGE*') {
            if ($sect) {
               $charts['CHART'][] = $sect;
            }
            $sect = array();
            $sect['TYPE'] = trim($line, '*');
        }
        else if ($line[0] == '@') {
            $lineparts = explode('=', substr($line, 1) , 2);
            if (count($lineparts) > 1) {
                $key = trim($lineparts[0]);
                $value = trim($lineparts[1]);

                if (in_array($key, $generallists)) {
                    if (!array_key_exists($key, $charts['GENERAL'])) {
                        $charts['GENERAL'][$key] = array();
                    }
                    $charts['GENERAL'][$key][] = $value;
                }
                else if ($sect && in_array($key, $sectlists[$sect['TYPE']])) {
                    if (!array_key_exists($key, $sect)) {
                        $sect[$key] = array();
                    }
                    $sect[$key][] = $value;
                }
                else {
                    return array('stat' => -1, 'msg' => 
                       'ERROR: Invalid list item key: ' . $key);
                }
            }            
            else {
                return array('stat' => -1, 'msg' => 
                   'ERROR: Invalid configuration file row: ' . $line);
            }
        }
        else {
            $lineparts = explode('=', $line, 2);
            if (count($lineparts) > 1) {
                $key = trim($lineparts[0]);
                $value = trim($lineparts[1]);

                if (in_array($key, $generalvalues)) {
                    $charts['GENERAL'][$key] = $value;
                }
                else if ($sect && (in_array($key, $sectvalues[$sect['TYPE']]) || preg_match('/^AXIS_[0-9]+$/', $key))) {
                    $sect[$key] = $value;
                }
                else {
                    return array('stat' => -1, 'msg' => 
                       'ERROR: Invalid configuration key: ' . $key);
                }
            }
            else {
                return array('stat' => -1, 'msg' => 
                   'ERROR: Invalid configuration file row: ' . $line);
            }
        }
    }

    if ($sect) $charts['CHART'][] = $sect;

    # muunnokset
    if (!array_key_exists('DB_TYPE', $charts['GENERAL'])) {
        return array('stat' => -1, 'msg' => 
                     'ERROR: Configuration missing DB_TYPE.');
    }
    $charts['GENERAL']['DB_TYPE'] = strtoupper($charts['GENERAL']['DB_TYPE']);
    
    foreach($charts['CHART'] as &$chart) {
        if ($chart['TYPE'] == 'CHART') {
	        if (!array_key_exists('TITLE', $chart)) {
	            $chart['TITLE'] = '';
	        }
	        
	        # tarkistetaan ykkosakseli
	        if (!array_key_exists('AXIS_1', $chart)) {
	            return array('stat' => -1, 'msg' => 
	               'ERROR: Chart missing AXIS_1 in configuration.');
	        }
	
	        # kasitellaan axikset
	        foreach(array_keys($chart) as $arraykey) {
	        	if (0 === strpos($arraykey, 'AXIS_')) {
	        		$temp = handleAXISConfRow($chart[$arraykey]);
	            	if (!$temp) {
	                	return array('stat' => -1, 'msg' => 
	                    	'ERROR: Invalid axis configuration value: ' . $chart[$arraykey]);
	            	}
	            	$chart[$arraykey] = $temp;	        		
	        	}
	        }

            if (array_key_exists('SIZE', $chart)) {
	            $temp = handleSIZEConfRow($chart['SIZE']);
	            if (!$temp) {
	                return array('stat' => -1, 'msg' => 
	                    'ERROR: Invalid size configuration value: ' . $chart['SIZE']);
	            }
	            $chart['SIZE'] = $temp;
	        } 
	        
	        if (!array_key_exists('SERIES', $chart)) {
	            return array('stat' => -1, 'msg' => 
	                'ERROR: Chart contains no SERIES.');
	        }
	
	        # tarkistetaan legend
	        if (!array_key_exists('LEGEND', $chart)) {
	            $chart['LEGEND'] = array();
	            $chart['LEGEND']['cols'] = 6;
	            $chart['LEGEND']['values'] = 0;
	        }
	        else {
	            # muunnetaan LEGEND arrayksi
	            $temp = handleLEGENDConfRow($chart['LEGEND']);
	            if (!$temp) {
	                return array('stat' => -1, 'msg' => 
	                    'ERROR: Invalid legend configuration value: ' . $chart['LEGEND']);
	            }
	            $chart['LEGEND'] = $temp;
	        }
	
	        foreach($chart['SERIES'] as &$ser) {
	            $temp = handleSERIESConfRow($ser);
	            if (!$temp) {
	                return array('stat' => -1, 'msg' => 
	                    'ERROR: Invalid SERIES configuration value: ' . $ser);
	            }
	            $ser = $temp;
	        }
	        unset($ser);
        }
        else if ($chart['TYPE'] == 'IMAGE') {
	        if (!array_key_exists('BACKGROUND', $chart)) {
	            return array('stat' => -1, 'msg' => 
	               'ERROR: Image missing BACKGROUND in configuration.');
	        }

        	if (array_key_exists('TITLE', $chart)) {
        		$temp = handleIMAGETITLEConfRow($chart['TITLE']);
	            if (!$temp) {
	                return array('stat' => -1, 'msg' => 
	                    'ERROR: Invalid IMAGE TITLE configuration value: ' . $chart['TITLE']);
	            }
	            $chart['TITLE'] = $temp;
	        }

	        if (!array_key_exists('POSITION', $chart)) {
	        	$chart['POSITION'] = array();
	        }
	        	        
	        foreach($chart['POSITION'] as &$pos) {
	            $temp = handlePOSITIONConfRow($pos);
	            if (!$temp) {
	                return array('stat' => -1, 'msg' => 
	                    'ERROR: Invalid POSITION configuration value: ' . $pos);
	            }
	            $pos = $temp;
	        }   
	        unset($pos);     

	        if (!array_key_exists('LABEL', $chart)) {
	        	$chart['LABEL'] = array();
	        } 
	       
	        foreach($chart['LABEL'] as &$label) {
	            $temp = handleLABELConfRow($label);
	            if (!$temp) {
	                return array('stat' => -1, 'msg' => 
	                    'ERROR: Invalid LABEL configuration value: ' . $label);
	            }
	            $label = $temp;
	        }     
	        unset($label);   
	    }
    }
    unset($chart);

    
    # tarkistukset
    $temp = $charts['GENERAL']['DB_TYPE'];
    if ($temp != 'MYSQL' && $temp != 'SQLITE') {
        return array('stat' => -1, 'msg' => 
                     'ERROR: Invalid DB_TYPE configuration value: ' . $temp);
    }
    foreach($charts['CHART'] as &$chart) {
        if ($chart['TYPE'] == 'CHART') {
	        # tarkistetaan onko kaikilla kakkosakseli, jos ei, niin kopioidaan
	        # ensimmainen
	        if (!array_key_exists('AXIS_2', $chart)) {
	            $chart['AXIS_2'] = $chart['AXIS_1'];
	        }

	        $has_axis = array();	        
	        $has_axis[] = 1;
	        foreach($chart['SERIES'] as &$ser) {
	            if (array_key_exists('axis', $ser)) {
	            	if (!in_array($ser['axis'], $has_axis)) $has_axis[] = $ser['axis'];
	            }	            
	        }
	        unset($ser);
	        sort($has_axis);
	        $chart['has_axis'] = $has_axis;

	        foreach ($has_axis as $axis) {
	        	$axiskey = sprintf('AXIS_%d', $axis);
	        	if (!array_key_exists($axiskey, $chart)) {
			        return array('stat' => -1, 'msg' => 
                     			 'ERROR: Missing AXIS-definition: ' . $axiskey);
	        	}
	        }	        
        }
    }
    unset($chart);
    if (!array_key_exists('CACHE_IMAGES', $charts['GENERAL'])) {
    	$charts['GENERAL']['CACHE_IMAGES'] = false;
    }
    else {
    	if ($charts['GENERAL']['CACHE_IMAGES'] == 'true') {
    		$charts['GENERAL']['CACHE_IMAGES'] = true;
    	}
    	else {
    		$charts['GENERAL']['CACHE_IMAGES'] = false;
    	}
    }

    if ($charts['GENERAL']['CACHE_IMAGES']) {
    	if (!array_key_exists('CACHE_DIRECTORY', $charts['GENERAL'])) {
    		return array('stat' => -1, 'msg' => 'ERROR: Caching enabled, but missing CACHE_DIRECTORY');    		
    	}

    	if (!array_key_exists('CACHE_URL_PATH', $charts['GENERAL'])) {
    		return array('stat' => -1, 'msg' => 'ERROR: Caching enabled, but missing CACHE_URL_PATH');
    	}
    	     	
    	$temp = $charts['GENERAL']['CACHE_DIRECTORY'];
		if (!is_dir($temp)) {
			return array('stat' => -1, 'msg' => 
					     'ERROR: Caching enabled, but CACHE_DIRECTORY not found: ' . $temp);						
    	}
    }
    
    return array('stat' => 0, 'msg' => '', 'result' => $charts);
}

?>
