<?PHP
###########################################################################
# 
# File:            taloLoggerGraph_htmlchart.inc (PHP)
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.2c
#
# Date:            14.02.2015
#
# Functions:       getHtmlChartPreDocument
#                  getHtmlChartDiv
#                  getHtmlImageDiv
#                  getColorHexString
#                  getHtmlColor
#                  getArrayColor
#                   
# Description:     House automation data logger graphing application for PHP. 
# 
#                  Support functions to produce D3 HTML/JavaScript charts.
#
# Requirements:    PHP 5.1 or newer (www.php.net)
#
# Version history: ** 02.01.2015 v1.2a (Olli Lammi) **
#                  First version. 
#
#                  ** 09.01.2015 v1.2b (Olli Lammi) **
#                  Released first version. 
#
#                  ** 14.02.2015 v1.2c (Olli Lammi) **
#                  Changed the way d3js is served.
#
###########################################################################

# Returns string to append once to the document body before any chart divisions.
function getHtmlChartPreDocument($isCmdline) {
    $css = '

    .tlgchart {
        font-family: Lucida Sans Typewriter,Lucida Console,monaco,Bitstream Vera Sans Mono,monospace;
        font-size: 10px;
    }

    .axis path,
    .axis line {
        fill: none;
        stroke: #000;
        shape-rendering: crispEdges;
    }

    .tlgchart_line {
        fill: none;
        stroke-width: 1.0px;
    }

    .tlgchart_area {
        stroke: none;
    }

    .tlgchart_grid .tick {
        stroke: #E0E0E0;
        opacity: 1.0;
        stroke-width: 1.0px;
    }

    .tlgchart_grid path {
        stroke-width: 0;
    }

    .tlgchart_grid2 .tick {
        stroke: #F0F0F0;
        opacity: 1.0;
        stroke-width: 1.0px;
    }

    .tlgchart_grid2 path {
        stroke-width: 0;
    }

    .tlgchart_legend rect {
        stroke: #000000;
        fill: #EBEBEB;
        stroke-width:1 ;
    }

    .tlgchart_title {
        font-family: Trebuchet MS,Lucida Grande,Lucida Sans Unicode,Lucida Sans,Tahoma,sans-serif;
        font-size: 12px;
    }

    .tlgchart_legend_title {
        font-family: Trebuchet MS,Lucida Grande,Lucida Sans Unicode,Lucida Sans,Tahoma,sans-serif;
        font-size: 11px;
    }

    .tlgchart_legend_line {
        stroke-width: 2;
    }

    .tlgchart_legend_area {
        stroke: #000000;
        stroke-width: 1;
    }

    .tlgchart_version {
        font-family: Lucida Sans Typewriter,Lucida Console,monaco,Bitstream Vera Sans Mono,monospace;
        font-size: 10px;
        fill: #DCDCDC;
    }

    .tlgimage_circle {
    }

    .tlgimage_text {
        font-family: Lucida Sans Typewriter,Lucida Console,monaco,Bitstream Vera Sans Mono,monospace;
        font-size: 10px;
    }

    ';

    $css = trim(preg_replace('/\s+/', ' ', $css));

    $d3js = 'http://d3js.org/d3.v3.js';
    if (!$isCmdline) {
	    # find the d3js library from local path, use network one if local one is not available
    	$d3js = findJSFile('d3.v3.js', 'http://d3js.org/d3.v3.js');
    }
    
	return '
        <SCRIPT type="text/javascript">
            var taloLoggerGraphStyle = document.createElement(\'style\');
            taloLoggerGraphStyle.type = \'text/css\';
            taloLoggerGraphStyle.innerHTML = \'' . $css . '\';
            document.getElementsByTagName(\'head\')[0].appendChild(taloLoggerGraphStyle);
        </SCRIPT>

        <SCRIPT SRC="' . $d3js . '" CHARSET="UTF-8" TYPE="text/javascript"></SCRIPT>
    ';
}

# Returns DIV-element with all data and scripts for this chart.
function getHtmlChartDiv(&$chart, $xsize, $ysize, $legendrows, $starttime, $endtime, $title, $axis_has_data, $axis_map) {	
	$htmlid = rand();
	$res = '
        <DIV CLASS="tlgchart" ID="tlgchart_#ID#"></DIV>
            <SCRIPT TYPE="text/javascript">
	
            var parseDate_#ID# = function(d) { return new Date(+d * 1000.0) };
    ';
		
	$res .= getHtmlChartSetupData($chart, $starttime, $endtime, $xsize, $ysize, $legendrows, $axis_has_data, $axis_map, $title);
	
    $res .= getHtmlChartJSONData($chart, $axis_map);
	
    $res .= getHtmlChartMargins($legendrows, $axis_has_data);

    $res .= getHtmlChartScalesAndAxises($chart, $axis_has_data, $axis_map);

    $res .= getHtmlChartSVG();
    
    $res .= getHtmlChartAddSeries();

    if ($legendrows > 0) {
        $res .= getHtmlChartLegend($chart, $axis_has_data, $axis_map, $legendrows);
    }
    
	$res .= "\n</SCRIPT>\n";
	
	return str_replace('#ID#', $htmlid, $res);
}

function getHtmlChartSetupData(&$chart, $starttime, $endtime, $xsize, $ysize, $legendrows, $axis_has_data, $axis_map, $title) {
    $LEGENDROWHEIGHT = 18;
			
	$res = '
        var setupdata_#ID# = {
            xsize: ' . $xsize . ',
            ysize: ' . ($ysize + $legendrows * $LEGENDROWHEIGHT) . ',
            legendrows: ' . $legendrows . ',
            legendcolumns: ' . $chart['LEGEND']['cols'] . ',	
            starttime: ' . $starttime . ',
            endtime: ' . $endtime . ',
            title: "' . $title . '",
	        axises: [
    ';
	
	foreach($axis_has_data as $axis) {
		$axisind = $axis_map[$axis];
		$axistitle = sprintf('AXIS_%d', $axis);
		$axisunit = $chart[$axistitle]['title'];
		if (count($axis_has_data) > 2 && $axisind > 1) {
			$axisunit .= ' (' . $axisind . ')';
		}
	
		if ($axisind > 1) $res .= ',';
		$res .= '
            { index: ' . $axisind . ',
              title: "' . $axisunit . '",
              min: ' . $chart[$axistitle]['min'] . ',
              max: ' . $chart[$axistitle]['max'] . ',
              minortick: ' . $chart[$axistitle]['mitick'] . ',
              majortick: ' . $chart[$axistitle]['matick'] . ' }
        ';
	}
		
	$res .= '
        ]
        };
    ';
		
	return $res;
}

function getHtmlChartJSONData(&$chart, $axis_map) {
	$seriesindexes = array();
	
	$cntseries = 0;
	foreach($chart['SERIES'] as &$ser) {
		if ($ser['hasdata']) {
			$cntseries += 1;
		}
	}	
	
	$ax = 1;
	$cntseries2 = 0;
	while ($cntseries2 < $cntseries) {
		$ind = 0;
		foreach($chart['SERIES'] as &$ser) {
			if ($ser['hasdata']) {
				$axis = 1;
				if (array_key_exists('axis', $ser)) {
					$axis = $axis_map[$ser['axis']];
				}

				if ($axis == $ax) {
					$seriesindexes[] = $ind; 
					$cntseries2 += 1;
				}
			}
			$ind += 1;
		}
		
		$ax += 1;
	}
		
	$res = 'var jsondata_#ID# = [';
	
	$i = 0;
	foreach($seriesindexes as $ind) {
		$ser = &$chart['SERIES'][$ind];

		$axis = 1;
		if (array_key_exists('axis', $ser)) {
			$axis = $axis_map[$ser['axis']];
		}
						
		if ($i > 0) $res .= ',';
		$res .= '{type:"' . $ser['type'] . '",';
		$res .= 'color:"' . $ser['color'] . '",';
		$res .= 'axis:"' . $axis . '",';
		$res .= 'legend:"' . $ser['legend_text'] . '",';
		$res .= 'data: [';
		$j = 0;
		$cnt = count($ser['XARRAY']);
		while ($j < $cnt) {
			if ($j > 0) $res .= ',';
			$res .= '{t:' . $ser['XARRAY'][$j] . ',v:' . $ser['YARRAY'][$j] . '}';
			$j += 1;
		}
		
		$res .= "]}\n";	
		$i += 1;			

		unset($ser);
	}
		
	$res .= '];';
	return $res;
}

function getHtmlChartMargins($legendrows, $axis_has_data) {
	$LEGENDROWHEIGHT = 18;

	$axismargin = 40;
	if (count($axis_has_data) > 2) {
		$axismargin += (count($axis_has_data) - 2) * 50;
	}
	
	$res = '
        var margin_#ID# = {top: 20, left: 40, bottom: ' . (74 + $legendrows * $LEGENDROWHEIGHT) . ', right: ' . $axismargin . '};
        var width_#ID# = setupdata_#ID#.xsize - margin_#ID#.left - margin_#ID#.right;
        var height_#ID# = setupdata_#ID#.ysize - margin_#ID#.top - margin_#ID#.bottom;
    ';
		
	return $res;
}

function getHtmlChartScalesAndAxises(&$chart, $axis_has_data, $axis_map) {
	$res = '
        var scale_x_#ID# = d3.time.scale()
            .domain([new Date(setupdata_#ID#.starttime * 1000.0), new Date(setupdata_#ID#.endtime * 1000.0)])
            .range([0,width_#ID#]);
	
        var xAxis_#ID# = d3.svg.axis()
            .scale(scale_x_#ID#)
            .ticks(d3.time.minutes, 60)
            .tickFormat(d3.time.format("%H:%M"))
            .orient("bottom");
    ';

	$majortick1 = 10;
	$minortick1 = 50;
	
	foreach($axis_has_data as $axis) {
		$axisind = $axis_map[$axis];
		$axistitle = sprintf('AXIS_%d', $axis);

		$majortick = ($chart[$axistitle]['max'] - $chart[$axistitle]['min']) / $chart[$axistitle]['matick'];
		
		if ($axisind == 1) {
			$majortick1 = $majortick;
			$minortick1 = ($chart[$axistitle]['max'] - $chart[$axistitle]['min']) / $chart[$axistitle]['mitick'];
		}
		
		$res .= '
            var scale_y_#ID#_' . $axisind . ' = d3.scale.linear()
                .domain([' . $chart[$axistitle]['min'] . ', ' . $chart[$axistitle]['max'] . '])
                .range([height_#ID#, 0]);		

            var yAxis_#ID#_' . $axisind . ' = d3.svg.axis()
            	.ticks(' . $majortick . ')	
                .scale(scale_y_#ID#_' . $axisind . ')
        ';
		
	    if ($axisind <= 1) {
	    	$res .= '        .orient("left");';
	    }
	    else {
	    	$res .= '        .orient("right");';
	    }
	    $res .= "\n";
	    
	    $res .= '
            var line_#ID#_' . $axisind . ' = d3.svg.line()
                .x(function(d) { return scale_x_#ID#(d.t); })
                .y(function(d) { return scale_y_#ID#_' . $axisind . '(d.v); });

            var area_#ID#_' . $axisind . ' = d3.svg.area()
                .interpolate("step")
                .x(function(d) { return scale_x_#ID#(d.t); })
                .y0(function(d) { return scale_y_#ID#_' . $axisind . '(' . $chart[$axistitle]['min'] . '); })
                .y1(function(d) { return scale_y_#ID#_' . $axisind . '(d.v); });   
	    ';
	}

	$res .= '
        var xGrid_#ID# = d3.svg.axis()
            .scale(scale_x_#ID#)
            .ticks(d3.time.minutes, 60)
            .tickFormat("")
            .tickSize(-height_#ID#, 0, 0)
            .orient("bottom");
	
        var xGrid2_#ID# = d3.svg.axis()
            .scale(scale_x_#ID#)
            .ticks(d3.time.minutes, 15)
            .tickFormat("")
            .tickSize(-height_#ID#, 0, 0)
            .orient("bottom");
	
        var yGrid_#ID# = d3.svg.axis()
            .scale(scale_y_#ID#_1)
            .ticks(' . $majortick1 . ')		
            .tickFormat("")
            .tickSize(-width_#ID#, 0, 0)
            .orient("left");
	
        var yGrid2_#ID# = d3.svg.axis()
            .scale(scale_y_#ID#_1)
            .ticks(' . $minortick1 . ')		
            .tickFormat("")
            .tickSize(-width_#ID#, 0, 0)
            .orient("left");
    ';
				
	return $res;
}

function getHtmlChartSVG() {
	$res = '
        var svg_#ID# = d3.select("#tlgchart_#ID#").append("svg")
            .attr("width", width_#ID# + margin_#ID#.left + margin_#ID#.right)
            .attr("height", height_#ID# + margin_#ID#.top + margin_#ID#.bottom)
          .append("g")
            .attr("transform", "translate(" + margin_#ID#.left + "," + margin_#ID#.top + ")");

        svg_#ID#.append("rect")
            .attr("transform", "translate(" + (-margin_#ID#.left) + "," + (-margin_#ID#.top) + ")")
            .attr("x", 0)
            .attr("y", 0)
            .attr("width", width_#ID# + margin_#ID#.left + margin_#ID#.right)
            .attr("height", height_#ID# + margin_#ID#.top + margin_#ID#.bottom)
            .style("stroke", "#000000")
            .style("fill", "none")
            .style("stroke-width", 1)

        svg_#ID#.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + height_#ID# + ")")
            .call(xAxis_#ID#)
            .selectAll("text")
              .style("text-anchor", "end")
              .attr("dx", "-8px").attr("dy", "-6px")
              .attr("transform", "rotate(-90)");

         svg_#ID#.append("text")			
           .attr("class", "tlgchart_title")
           .attr("transform", "translate(" + (width_#ID# / 2) + "," + (height_#ID# + margin_#ID#.bottom - 10) + ")")
           .style("text-anchor", "middle")
           .text(setupdata_#ID#.title);

         svg_#ID#.append("text")			
           .attr("class", "tlgchart_version")
           .attr("transform", "translate(" + (width_#ID# + margin_#ID#.right - 5) + "," + (height_#ID# + margin_#ID#.bottom - 5) + ")")
           .style("text-anchor", "end")
           .text("taloLoggerGraph ' . getVersion() . '");
			
        svg_#ID#.append("g")
            .attr("class", "tlgchart_grid2")
            .attr("transform", "translate(0," + height_#ID# + ")")
            .call(xGrid2_#ID#);

        svg_#ID#.append("g")
            .attr("class", "tlgchart_grid2")
            .call(yGrid2_#ID#);
	
        svg_#ID#.append("g")
            .attr("class", "tlgchart_grid")
            .call(yGrid_#ID#);
	
        svg_#ID#.append("g")
            .attr("class", "tlgchart_grid")
            .attr("transform", "translate(0," + height_#ID# + ")")
            .call(xGrid_#ID#);
			
        setupdata_#ID#.axises.forEach(function(d) {
            var tempfunc = eval("yAxis_#ID#_" + d.index);
            if (d.index == 1) {
                svg_#ID#.append("g")
                    .attr("class", "y axis")
                    .call(tempfunc)
                    .append("text")
                      .attr("transform", "translate(0," + (height_#ID# / 2)+ "),rotate(-90)")
                      .attr("dy", "-29")
                      .style("text-anchor", "middle")
                      .text(d.title);				
            } else {
                svg_#ID#.append("g")
                    .attr("class", "y axis")
                    .attr("transform", "translate(" + (width_#ID# + (d.index - 2) * 50) + ",0)")
                    .call(tempfunc)
                    .append("text")
                      .attr("transform", "translate(0," + (height_#ID# / 2)+ "),rotate(-90)")
                      .attr("dy", "35")
                      .style("text-anchor", "middle")
                      .text(d.title);
            }
        });
    ';
		
	return $res;
}

function getHtmlChartAddSeries() {
	$res = '
        jsondata_#ID#.forEach(function(d) {
            d.data.forEach(function(d2) {
                d2.t = parseDate_#ID#(d2.t);
            });
	
            var lineData = null;
    		var areaData = null;
            if (d.type == "STATE") {
                var tempfunc = eval("area_#ID#_" + d.axis);
                areaData = tempfunc(d.data);
            } else {
                var tempfunc = eval("line_#ID#_" + d.axis);
                lineData = tempfunc(d.data);
            }
	    
            if (areaData != null) {
                svg_#ID#.append("path")
               .attr("class", "tlgchart_area")
               .style("fill", function() {
                   return d.color; })
               .attr("d", areaData);
            }

            if (lineData != null) {
                svg_#ID#.append("path")
                .attr("class", "tlgchart_line")
                .style("stroke", function() {
                    return d.color; })
                .attr("d", lineData);
            }
        });
	';
	
	return $res;
}

function getHtmlChartLegend(&$chart, $axis_has_data, $axis_map, $legendrows) {
    $res = '
        var legend_#ID# = svg_#ID#.append("g")
            .attr("class", "tlgchart_legend")
            .attr("transform", "translate(0," + (height_#ID# + 42) + ")")
            .attr("height", setupdata_#ID#.legendrows * 18 + 2)
            .attr("width", setupdata_#ID#.xsize - 80);

        legend_#ID#.append("rect")
            .attr("x", 0)
            .attr("y", 0)
            .attr("width", setupdata_#ID#.xsize - 80)
            .attr("height", setupdata_#ID#.legendrows * 18 + 2);

        legend_#ID#.append("line")
            .attr("x1", 4)
            .attr("x2", setupdata_#ID#.xsize - 80 + 2)
            .attr("y1", setupdata_#ID#.legendrows * 18 + 3)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 3)
            .style("stroke", "#646464");

        legend_#ID#.append("line")
            .attr("x1", 4)
            .attr("x2", setupdata_#ID#.xsize - 80 + 3)
            .attr("y1", setupdata_#ID#.legendrows * 18 + 4)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 4)
            .style("stroke", "#8B8B8B");

        legend_#ID#.append("line")
            .attr("x1", 4)
            .attr("x2", setupdata_#ID#.xsize - 80 + 4)
            .attr("y1", setupdata_#ID#.legendrows * 18 + 5)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 5)
            .style("stroke", "#B2B2B2");

        legend_#ID#.append("line")
            .attr("x1", 4)
            .attr("x2", setupdata_#ID#.xsize - 80 + 5)
            .attr("y1", setupdata_#ID#.legendrows * 18 + 6)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 6)
            .style("stroke", "#D7D7D7");

        legend_#ID#.append("line")
            .attr("x1", setupdata_#ID#.xsize - 80 + 1)
            .attr("x2", setupdata_#ID#.xsize - 80 + 1)
            .attr("y1", 4)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 3)
            .style("stroke", "#646464");

        legend_#ID#.append("line")
            .attr("x1", setupdata_#ID#.xsize - 80 + 2)
            .attr("x2", setupdata_#ID#.xsize - 80 + 2)
            .attr("y1", 4)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 4)
            .style("stroke", "#8B8B8B");

        legend_#ID#.append("line")
            .attr("x1", setupdata_#ID#.xsize - 80 + 3)
            .attr("x2", setupdata_#ID#.xsize - 80 + 3)
            .attr("y1", 4)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 5)
            .style("stroke", "#B2B2B2");

        legend_#ID#.append("line")
            .attr("x1", setupdata_#ID#.xsize - 80 + 4)
            .attr("x2", setupdata_#ID#.xsize - 80 + 4)
            .attr("y1", 4)
            .attr("y2", setupdata_#ID#.legendrows * 18 + 6)
            .style("stroke", "#D7D7D7");
 
        var rows = setupdata_#ID#.legendrows;
        var cols = setupdata_#ID#.legendcolumns;
    	if (cols > jsondata_#ID#.length) {
    		cols = jsondata_#ID#.length;
    	}
        var legenddx = Math.floor((setupdata_#ID#.xsize - 100) / cols);

        jsondata_#ID#.forEach(function(d, i) {
            var r = Math.floor(i / cols);
            var c = i % cols;
            legend_#ID#.append("text")
                .attr("class", "tlgchart_legend_title")
                .attr("transform", "translate(0,0)")
                .attr("dx", 36 + c * legenddx)
                .attr("dy", ((r + 1) * 18) - 4)
                .style("text-anchor", "start")
                .text(d.legend);

            if (d.type == "STATE") {
                legend_#ID#.append("rect")
                    .attr("class", "tlgchart_legend_area")
                    .attr("transform", "translate(" + (13 + c * legenddx) + "," + ((r + 1) * 18 - 13) + ")")
                    .attr("width", 11)
                    .attr("height", 11)
                    .style("fill", d.color);
            } else {
                legend_#ID#.append("line")
                    .attr("class", "tlgchart_legend_line")
                    .attr("transform", "translate(" + (10 + c * legenddx) + "," + ((r + 1) * 18 - 7) + ")")
                    .attr("x1", 0)
                    .attr("x2", 17)
                    .attr("y1", 0)
                    .attr("y2", 0)
                    .style("stroke", d.color);
            }
        });
	
    ';

    return $res;
}

# Returns DIV-element with all data and scripts for this image.
function getHtmlImageDiv(&$chart, &$imageitems) {	
    $htmlid = rand();
    $res = '
        <DIV CLASS="tlgimage" ID="tlgchart_#ID#"><IMG SRC="##IMAGE##URL##" ID="tlgimage_#ID#"></DIV>
            <SCRIPT TYPE="text/javascript">            
    ';

    $res .= getHtmlImageJSON($imageitems);		

    $res .= getHtmlImageSVG();
	
    $res .= getHtmlImageItemProcess();

    $res .= "\n</SCRIPT>\n";
	
    return str_replace('#ID#', $htmlid, $res);
}

function getHtmlImageJSON(&$imageitems) {
	$res = 'var imagedata_#ID# = [';

        $i = 0;       	
	foreach($imageitems as $item) {
            $colstr = getHtmlColor($item['color']);
            if ($i > 0) $res .= ',';
		$res .= '
            { type: "' . $item['type'] . '",
              x: ' . $item['x'] . ',
              y: ' . $item['y'] . ',
              color: "' . $colstr . '",
              data: "' . $item['data'] . '" }';
            $i += 1;
	}
		
	$res .= '];';
		
	return $res;
}

function getHtmlImageSVG() {
	$res = '

        var img_#ID# = d3.select("#tlgimage_#ID#");
        var svg_#ID# = null;
        img_#ID#.on("load", function() { setupImage_#ID#(); });

        function setupImage_#ID#() { 
           var iheight = img_#ID#.property("height");
           var iwidth = img_#ID#.property("width");
           img_#ID#.remove();

            svg_#ID# = d3.select("#tlgchart_#ID#").append("svg")
              .attr("width", iwidth)
              .attr("height",iheight)
              .append("g")
                .attr("transform", "translate(0,0)");

            svg_#ID#.append("image")
                .attr("transform", "translate(0,0)")
                .attr("x", 0)
                .attr("y", 0)
                .attr("width", iwidth)
                .attr("height", iheight)
	        .attr("xlink:href", "##IMAGE##URL##");

             svg_#ID#.append("text")			
               .attr("class", "tlgchart_version")
               .attr("transform", "translate(0,0)")
               .attr("x", (iwidth - 5))
               .attr("y", (iheight - 5))
               .style("text-anchor", "end")
               .text("taloLoggerGraph ' . getVersion() . '");

            addImageItems_#ID#();
        }
    ';
	
	return $res;
}

function getHtmlImageItemProcess() {
    $res = '
        function addImageItems_#ID#() {
            imagedata_#ID#.forEach(function(d) {
                if (d.type == "text") {
                    svg_#ID#.append("text")
                        .attr("class", "tlgimage_text")
                        .attr("transform", "translate(0,0)")
                        .attr("x", d.x)
                        .attr("y", (d.y + 11))
                        .style("fill", d.color)
                        .style("text-anchor", "start")
                        .text(d.data);
                }
                else if (d.type == "circle") {
                    svg_#ID#.append("circle")
                        .attr("class", "tlgimage_circle")
                        .attr("transform", "translate(0,0)")
                        .attr("cx", (d.x + 1))
                        .attr("cy", d.y)
                        .attr("r", (d.data / 2))
                        .style("fill", d.color);
                }
            });
        }
    ';

    return $res;
}

$COLORCONSTANTS = array(
    'ALICEBLUE' => '#F0F8FF',
    'ANTIQUEWHITE' => '#FAEBD7',
    'ANTIQUEWHITE1' => '#FFEFDB',
    'ANTIQUEWHITE2' => '#EEDFCC',
    'ANTIQUEWHITE3' => '#CDC0B0',
    'ANTIQUEWHITE4' => '#8B8378',
    'AQUA' => '#00FFFF',
    'AQUAMARINE' => '#7FFFD4',
    'AQUAMARINE1' => '#7FFFD4',
    'AQUAMARINE2' => '#76EEC6',
    'AQUAMARINE3' => '#66CDAA',
    'AQUAMARINE4' => '#458B74',
    'AZURE' => '#F0FFFF',
    'AZURE1' => '#F0FFFF',
    'AZURE2' => '#E0EEEE',
    'AZURE3' => '#C1CDCD',
    'AZURE4' => '#838B8B',
    'BEIGE' => '#F5F5DC',
    'BISQUE' => '#FFE4C4',
    'BISQUE1' => '#FFE4C4',
    'BISQUE2' => '#EED5B7',
    'BISQUE3' => '#CDB79E',
    'BISQUE4' => '#8B7D6B',
    'BLACK' => '#000000',
    'BLANCHEDALMOND' => '#FFEBCD',
    'BLUE' => '#0000FF',
    'BLUEVIOLET' => '#8A2BE2',
    'BROWN' => '#A52A2A',
    'BROWN1' => '#FF4040',
    'BROWN2' => '#EE3B3B',
    'BROWN3' => '#CD3333',
    'BROWN4' => '#8B2323',
    'BURLYWOOD' => '#DEB887',
    'BURLYWOOD1' => '#FFD39B',
    'BURLYWOOD2' => '#EEC591',
    'BURLYWOOD3' => '#CDAA7D',
    'BURLYWOOD4' => '#8B7355',
    'CADETBLUE' => '#5F9EA0',
    'CADETBLUE1' => '#98F5FF',
    'CADETBLUE2' => '#8EE5EE',
    'CADETBLUE3' => '#7AC5CD',
    'CADETBLUE4' => '#53868B',
    'CHARTREUSE' => '#7FFF00',
    'CHARTREUSE1' => '#7FFF00',
    'CHARTREUSE2' => '#76EE00',
    'CHARTREUSE3' => '#66CD00',
    'CHARTREUSE4' => '#458B00',
    'CHOCOLATE' => '#D2691E',
    'CHOCOLATE1' => '#FF7F24',
    'CHOCOLATE2' => '#EE7621',
    'CHOCOLATE3' => '#CD661D',
    'CHOCOLATE4' => '#8B4513',
    'CORAL' => '#FF7F50',
    'CORAL1' => '#FF7256',
    'CORAL2' => '#EE6A50',
    'CORAL3' => '#CD5B45',
    'CORAL4' => '#8B3E2F',
    'CORNFLOWERBLUE' => '#6495ED',
    'CORNSILK' => '#FFF8DC',
    'CRIMSON' => '#DC143C',
    'CYAN' => '#00FFFF',
    'CYAN1' => '#00FFFF',
    'CYAN2' => '#00EEEE',
    'CYAN3' => '#00CDCD',
    'CYAN4' => '#008B8B',
    'DARKBLUE' => '#00008B',
    'DARKCYAN' => '#008B8B',
    'DARKGOLDENROD' => '#B8860B',
    'DARKGOLDENROD1' => '#FFB90F',
    'DARKGOLDENROD2' => '#EEAD0E',
    'DARKGOLDENROD3' => '#CD950C',
    'DARKGOLDENROD4' => '#8B6508',
    'DARKGRAY' => '#646464',
    'DARKGREEN' => '#006400',
    'DARKKHAKI' => '#BDB76B',
    'DARKMAGENTA' => '#8B008B',
    'DARKOLIVEGREEN' => '#556B2F',
    'DARKOLIVEGREEN1' => '#CAFF70',
    'DARKOLIVEGREEN2' => '#BCEE68',
    'DARKOLIVEGREEN3' => '#A2CD5A',
    'DARKOLIVEGREEN4' => '#6E8B3D',
    'DARKORANGE' => '#FF8C00',
    'DARKORANGE1' => '#FF7F00',
    'DARKORANGE2' => '#EE7600',
    'DARKORANGE3' => '#CD6600',
    'DARKORANGE4' => '#8B4500',
    'DARKORCHID' => '#9932CC',
    'DARKORCHID1' => '#BF3EFF',
    'DARKORCHID2' => '#B23AEE',
    'DARKORCHID3' => '#9A32CD',
    'DARKORCHID4' => '#68228B',
    'DARKRED' => '#8B0000',
    'DARKSALMON' => '#E9967A',
    'DARKSEAGREEN' => '#8FBC8F',
    'DARKSEAGREEN1' => '#C1FFC1',
    'DARKSEAGREEN2' => '#B4EEB4',
    'DARKSEAGREEN3' => '#9BCD9B',
    'DARKSEAGREEN4' => '#698B69',
    'DARKSLATEBLUE' => '#483D8B',
    'DARKSLATEGRAY' => '#2F4F4F',
    'DARKSLATEGRAY1' => '#97FFFF',
    'DARKSLATEGRAY2' => '#8DEEEE',
    'DARKSLATEGRAY3' => '#79CDCD',
    'DARKSLATEGRAY4' => '#528B8B',
    'DARKTURQUOISE' => '#00CED1',
    'DARKVIOLET' => '#9400D3',
    'DEEPINDIGO' => '#8A2BE2',
    'DEEPPINK' => '#FF1493',
    'DEEPPINK1' => '#FF1493',
    'DEEPPINK2' => '#EE1289',
    'DEEPPINK3' => '#CD1076',
    'DEEPPINK4' => '#8B0A50',
    'DEEPSKYBLUE' => '#00BFFF',
    'DEEPSKYBLUE1' => '#00BFFF',
    'DEEPSKYBLUE2' => '#00B2EE',
    'DEEPSKYBLUE3' => '#009ACD',
    'DEEPSKYBLUE4' => '#00688B',
    'DIMGRAY' => '#696969',
    'DODGERBLUE' => '#1E90FF',
    'DODGERBLUE1' => '#1E90FF',
    'DODGERBLUE2' => '#1C86EE',
    'DODGERBLUE3' => '#1874CD',
    'DODGERBLUE4' => '#104E8B',
    'EGGPLANT' => '#90B0A8',
    'ELECTRICINDIGO' => '#6600FF',
    'FIREBRICK' => '#B22222',
    'FIREBRICK1' => '#FF3030',
    'FIREBRICK2' => '#EE2C2C',
    'FIREBRICK3' => '#CD2626',
    'FIREBRICK4' => '#8B1A1A',
    'FLORALWHITE' => '#FFFAF0',
    'FORESTGREEN' => '#228B22',
    'FUCHSIA' => '#FF00FF',
    'GAINSBORO' => '#DCDCDC',
    'GHOSTWHITE' => '#F8F8FF',
    'GOLD' => '#FFD700',
    'GOLD1' => '#FFD700',
    'GOLD2' => '#EEC900',
    'GOLD3' => '#CDAD00',
    'GOLD4' => '#8B7500',
    'GOLDENROD' => '#DAA520',
    'GOLDENROD1' => '#FFC125',
    'GOLDENROD2' => '#EEB422',
    'GOLDENROD3' => '#CD9B1D',
    'GOLDENROD4' => '#8B6914',
    'GRAY' => '#BEBEBE',
    'GRAY1' => '#0A0A0A',
    'GRAY2' => '#28281E',
    'GRAY3' => '#464646',
    'GRAY4' => '#646464',
    'GRAY5' => '#828282',
    'GRAY6' => '#A0A0A0',
    'GRAY7' => '#BEBEBE',
    'GRAY8' => '#D2D2D2',
    'GRAY9' => '#F0F0F0',
    'GREEN' => '#00FF00',
    'GREENYELLOW' => '#ADFF2F',
    'HONEYDEW' => '#C1CDC1',
    'HOTPINK' => '#FF69B4',
    'HOTPINK1' => '#FF6EB4',
    'HOTPINK2' => '#EE6AA7',
    'HOTPINK3' => '#CD6090',
    'HOTPINK4' => '#8B3A62',
    'HTMLDARKGREY' => '#A9A9A9',
    'HTMLGRAY' => '#808080',
    'HTMLGREEN' => '#008000',
    'HTMLHONEYDEW' => '#F0FFF0',
    'HTMLLIGHTYELLOW' => '#FFFFE0',
    'HTMLMAROON' => '#800000',
    'HTMLPURPLE' => '#800080',
    'INDIANRED' => '#CD5C5C',
    'INDIANRED1' => '#FF6A6A',
    'INDIANRED2' => '#EE6363',
    'INDIANRED3' => '#CD5555',
    'INDIANRED4' => '#8B3A3A',
    'INDIGO' => '#4B0082',
    'INDIGODYE' => '#00416A',
    'IVORY' => '#FFFFF0',
    'IVORY1' => '#FFFFF0',
    'IVORY2' => '#EEEEE0',
    'IVORY3' => '#CDCDC1',
    'IVORY4' => '#8B8B83',
    'KHAKI' => '#F0E68C',
    'KHAKI1' => '#FFF68F',
    'KHAKI2' => '#EEE685',
    'KHAKI3' => '#CDC673',
    'KHAKI4' => '#8B864E',
    'LAVENDER' => '#E6E6FA',
    'LAVENDERBLUSH' => '#FFF0F5',
    'LAVENDERBLUSH1' => '#FFF0F5',
    'LAVENDERBLUSH2' => '#EEE0E5',
    'LAVENDERBLUSH3' => '#CDC1C5',
    'LAVENDERBLUSH4' => '#8B8386',
    'LAWNGREEN' => '#7CFC00',
    'LEMONCHIFFON' => '#FFFACD',
    'LEMONCHIFFON1' => '#FFFACD',
    'LEMONCHIFFON2' => '#EEE9BF',
    'LEMONCHIFFON3' => '#CDC9A5',
    'LEMONCHIFFON4' => '#8B8970',
    'LIGHTBLUE' => '#ADD8E6',
    'LIGHTBLUE1' => '#BFEFFF',
    'LIGHTBLUE2' => '#B2DFEE',
    'LIGHTBLUE3' => '#9AC0CD',
    'LIGHTBLUE4' => '#68838B',
    'LIGHTCORAL' => '#F08080',
    'LIGHTCYAN' => '#E0FFFF',
    'LIGHTCYAN1' => '#E0FFFF',
    'LIGHTCYAN2' => '#D1EEEE',
    'LIGHTCYAN3' => '#B4CDCD',
    'LIGHTCYAN4' => '#7A8B8B',
    'LIGHTGOLDENROD' => '#EEDD82',
    'LIGHTGOLDENROD1' => '#FFEC8B',
    'LIGHTGOLDENROD2' => '#EEDC82',
    'LIGHTGOLDENROD3' => '#CDBE70',
    'LIGHTGOLDENROD4' => '#8B814C',
    'LIGHTGOLDENRODYELLOW' => '#FAFAD2',
    'LIGHTGRAY' => '#D3D3D3',
    'LIGHTGREEN' => '#90EE90',
    'LIGHTPINK' => '#FFB6C1',
    'LIGHTPINK1' => '#FFAEB9',
    'LIGHTPINK2' => '#EEA2AD',
    'LIGHTPINK3' => '#CD8C95',
    'LIGHTPINK4' => '#8B5F65',
    'LIGHTRED' => '#D3A7A8',
    'LIGHTSALMON' => '#FFA07A',
    'LIGHTSALMON1' => '#FFA07A',
    'LIGHTSALMON2' => '#EE9572',
    'LIGHTSALMON3' => '#CD8162',
    'LIGHTSALMON4' => '#8B5742',
    'LIGHTSEAGREEN' => '#20B2AA',
    'LIGHTSKYBLUE' => '#87CEFA',
    'LIGHTSKYBLUE1' => '#B0E2FF',
    'LIGHTSKYBLUE2' => '#A4D3EE',
    'LIGHTSKYBLUE3' => '#8DB6CD',
    'LIGHTSKYBLUE4' => '#607B8B',
    'LIGHTSLATEBLUE' => '#8470FF',
    'LIGHTSLATEGRAY' => '#778899',
    'LIGHTSTEELBLUE' => '#B0C4DE',
    'LIGHTSTEELBLUE1' => '#CAE1FF',
    'LIGHTSTEELBLUE2' => '#BCD2EE',
    'LIGHTSTEELBLUE3' => '#A2B5CD',
    'LIGHTSTEELBLUE4' => '#6E7B8B',
    'LIGHTYELLOW' => '#FFFFC8',
    'LIME' => '#00FF00',
    'LIMEGREEN' => '#32CD32',
    'LINEN' => '#FAF0E6',
    'MAGENTA' => '#FF00FF',
    'MAGENTA1' => '#FF00FF',
    'MAGENTA2' => '#EE00EE',
    'MAGENTA3' => '#CD00CD',
    'MAGENTA4' => '#8B008B',
    'MAROON' => '#B03060',
    'MAROON1' => '#FF34B3',
    'MAROON2' => '#EE30A7',
    'MAROON3' => '#CD2990',
    'MAROON4' => '#8B1C62',
    'MEDIUMAQUAMARINE' => '#66CDAA',
    'MEDIUMBLUE' => '#0000CD',
    'MEDIUMORCHID' => '#BA55D3',
    'MEDIUMORCHID1' => '#E066FF',
    'MEDIUMORCHID2' => '#D15FEE',
    'MEDIUMORCHID3' => '#B452CD',
    'MEDIUMORCHID4' => '#7A378B',
    'MEDIUMPURPLE' => '#9370DB',
    'MEDIUMPURPLE1' => '#AB82FF',
    'MEDIUMPURPLE2' => '#9F79EE',
    'MEDIUMPURPLE3' => '#8968CD',
    'MEDIUMPURPLE4' => '#5D478B',
    'MEDIUMRED' => '#8C2222',
    'MEDIUMSEAGREEN' => '#3CB371',
    'MEDIUMSLATEBLUE' => '#7B68EE',
    'MEDIUMSPRINGGREEN' => '#00FA9A',
    'MEDIUMTURQUOISE' => '#48D1CC',
    'MEDIUMVIOLETRED' => '#C71585',
    'MIDNIGHTBLUE' => '#191970',
    'MINTCREAM' => '#F5FFFA',
    'MISTYROSE' => '#FFE4E1',
    'MISTYROSE1' => '#FFE4E1',
    'MISTYROSE2' => '#EED5D2',
    'MISTYROSE3' => '#CDB7B5',
    'MISTYROSE4' => '#8B7D7B',
    'MOCCASIN' => '#FFE4B5',
    'NAVAJOWHITE' => '#FFDEAD',
    'NAVAJOWHITE1' => '#FFDEAD',
    'NAVAJOWHITE2' => '#EECFA1',
    'NAVAJOWHITE3' => '#CDB38B',
    'NAVAJOWHITE4' => '#8B795E',
    'NAVY' => '#000080',
    'OLDLACE' => '#FDF5E6',
    'OLIVE' => '#808000',
    'OLIVEDRAB' => '#6B8E23',
    'OLIVEDRAB1' => '#C0FF3E',
    'OLIVEDRAB2' => '#B3EE3A',
    'OLIVEDRAB3' => '#9ACD32',
    'OLIVEDRAB4' => '#698B22',
    'ORANGE' => '#FFA500',
    'ORANGE1' => '#FFA500',
    'ORANGE2' => '#EE9A00',
    'ORANGE3' => '#CD8500',
    'ORANGE4' => '#8B5A00',
    'ORANGERED' => '#FF4500',
    'ORANGERED1' => '#FF4500',
    'ORANGERED2' => '#EE4000',
    'ORANGERED3' => '#CD3700',
    'ORANGERED4' => '#8B2500',
    'ORCHID' => '#DA70D6',
    'ORCHID1' => '#FF83FA',
    'ORCHID2' => '#EE7AE9',
    'ORCHID3' => '#CD69C9',
    'ORCHID4' => '#8B4789',
    'PALEGOLDENROD' => '#EEE8AA',
    'PALEGREEN' => '#98FB98',
    'PALEGREEN1' => '#9AFF9A',
    'PALEGREEN2' => '#90EE90',
    'PALEGREEN3' => '#7CCD7C',
    'PALEGREEN4' => '#548B54',
    'PALETURQUOISE' => '#AFEEEE',
    'PALETURQUOISE1' => '#BBFFFF',
    'PALETURQUOISE2' => '#AEEEEE',
    'PALETURQUOISE3' => '#96CDCD',
    'PALETURQUOISE4' => '#668B8B',
    'PALEVIOLETRED' => '#DB7093',
    'PALEVIOLETRED1' => '#FF82AB',
    'PALEVIOLETRED2' => '#EE799F',
    'PALEVIOLETRED3' => '#CD6889',
    'PALEVIOLETRED4' => '#8B475D',
    'PAPAYAWHIP' => '#FFEFD5',
    'PEACHPUFF' => '#FFDAB9',
    'PEACHPUFF1' => '#FFDAB9',
    'PEACHPUFF2' => '#EECBAD',
    'PEACHPUFF3' => '#CDAF95',
    'PEACHPUFF4' => '#8B7765',
    'PERU' => '#CD853F',
    'PIGMENTINDIGO' => '#4B0082',
    'PINK' => '#FFC0CB',
    'PINK1' => '#FFB5C5',
    'PINK2' => '#EEA9B8',
    'PINK3' => '#CD919E',
    'PINK4' => '#8B636C',
    'PLUM' => '#DDA0DD',
    'PLUM1' => '#FFBBFF',
    'PLUM2' => '#EEAEEE',
    'PLUM3' => '#CD96CD',
    'PLUM4' => '#8B668B',
    'POWDERBLUE' => '#B0E0E6',
    'PURPLE' => '#A020F0',
    'PURPLE1' => '#9B30FF',
    'PURPLE2' => '#912CEE',
    'PURPLE3' => '#7D26CD',
    'PURPLE4' => '#551A8B',
    'RED' => '#FF0000',
    'ROSYBROWN' => '#BC8F8F',
    'ROSYBROWN1' => '#FFC1C1',
    'ROSYBROWN2' => '#EEB4B4',
    'ROSYBROWN3' => '#CD9B9B',
    'ROSYBROWN4' => '#8B6969',
    'ROYALBLUE' => '#4169E1',
    'ROYALBLUE1' => '#4876FF',
    'ROYALBLUE2' => '#436EEE',
    'ROYALBLUE3' => '#3A5FCD',
    'ROYALBLUE4' => '#27408B',
    'SADDLEBROWN' => '#8B4513',
    'SALMON' => '#FA8072',
    'SALMON1' => '#FF8C69',
    'SALMON2' => '#EE8262',
    'SALMON3' => '#CD7054',
    'SALMON4' => '#8B4C39',
    'SANDYBROWN' => '#F4A460',
    'SEAGREEN' => '#2E8B57',
    'SEAGREEN1' => '#54FF9F',
    'SEAGREEN2' => '#4EEE94',
    'SEAGREEN3' => '#43CD80',
    'SEAGREEN4' => '#2E8B57',
    'SEASHELL' => '#FFF5EE',
    'SEASHELL1' => '#FFF5EE',
    'SEASHELL2' => '#EEE5DE',
    'SEASHELL3' => '#CDC5BF',
    'SEASHELL4' => '#8B8682',
    'SIENNA' => '#A0522D',
    'SIENNA1' => '#FF8247',
    'SIENNA2' => '#EE7942',
    'SIENNA3' => '#CD6839',
    'SIENNA4' => '#8B4726',
    'SILVER' => '#C0C0C0',
    'SKYBLUE' => '#87CEEB',
    'SKYBLUE1' => '#87CEFF',
    'SKYBLUE2' => '#7EC0EE',
    'SKYBLUE3' => '#6CA6CD',
    'SKYBLUE4' => '#4A708B',
    'SLATEBLUE' => '#6A5ACD',
    'SLATEBLUE1' => '#836FFF',
    'SLATEBLUE2' => '#7A67EE',
    'SLATEBLUE3' => '#6959CD',
    'SLATEBLUE4' => '#473C8B',
    'SLATEGRAY' => '#708090',
    'SLATEGRAY1' => '#C6E2FF',
    'SLATEGRAY2' => '#B9D3EE',
    'SLATEGRAY3' => '#9FB6CD',
    'SLATEGRAY4' => '#6C7B8B',
    'SNOW' => '#FFFAFA',
    'SNOW1' => '#FFFAFA',
    'SNOW2' => '#EEE9E9',
    'SNOW3' => '#CDC9C9',
    'SNOW4' => '#8B8989',
    'SPRINGGREEN' => '#00FF7F',
    'SPRINGGREEN1' => '#00FF7F',
    'SPRINGGREEN2' => '#00EE76',
    'SPRINGGREEN3' => '#00CD66',
    'SPRINGGREEN4' => '#008B45',
    'STEELBLUE' => '#4682B4',
    'STEELBLUE1' => '#63B8FF',
    'STEELBLUE2' => '#5CACEE',
    'STEELBLUE3' => '#4F94CD',
    'STEELBLUE4' => '#36648B',
    'TAN' => '#D2B48C',
    'TAN1' => '#FFA54F',
    'TAN2' => '#EE9A49',
    'TAN3' => '#CD853F',
    'TAN4' => '#8B5A2B',
    'TEAL' => '#008080',
    'THISTLE' => '#D8BFD8',
    'THISTLE1' => '#FFE1FF',
    'THISTLE2' => '#EED2EE',
    'THISTLE3' => '#CDB5CD',
    'THISTLE4' => '#8B7B8B',
    'TOMATO' => '#FF6347',
    'TOMATO1' => '#FF6347',
    'TOMATO2' => '#EE5C42',
    'TOMATO3' => '#CD4F39',
    'TOMATO4' => '#8B3626',
    'TURQUOISE' => '#40E0D0',
    'TURQUOISE1' => '#00F5FF',
    'TURQUOISE2' => '#00E5EE',
    'TURQUOISE3' => '#00C5CD',
    'TURQUOISE4' => '#00868B',
    'VIOLET' => '#EE82EE',
    'VIOLETRED' => '#D02090',
    'VIOLETRED1' => '#FF3E96',
    'VIOLETRED2' => '#EE3A8C',
    'VIOLETRED3' => '#CD3278',
    'VIOLETRED4' => '#8B2252',
    'WHEAT' => '#F5DEB3',
    'WHEAT1' => '#FFE7BA',
    'WHEAT2' => '#EED8AE',
    'WHEAT3' => '#CDBA96',
    'WHEAT4' => '#8B7E66',
    'WHITE' => '#FFFFFF',
    'WHITESMOKE' => '#F5F5F5',
    'YELLOW' => '#FFFF00',
    'YELLOW1' => '#FFFF00',
    'YELLOW2' => '#EEEE00',
    'YELLOW3' => '#CDCD00',
    'YELLOW4' => '#8B8B00',
    'YELLOWGREEN' => '#9ACD32'
);

// returns a color in #XXXXXX hex string format corresponding to parameter string
function getColorHexString($str) {
	global $COLORCONSTANTS;
	
	$res = strtoupper(trim($str));
	if (strlen($res) > 0 && $res[0] == '#') return $res;
	
	return $COLORCONSTANTS[$res];
}

function getHtmlColor($color) {
	return sprintf('#%02X%02X%02X', $color[0], $color[1], $color[2]);
}

function getArrayColor($str) {
    $a = 0;
    $b = 0;
    $c = 0;
    
    if ($str[0] == '#') {
        sscanf(strtoupper(trim($str)), "#%02X%02X%02X", $a, $b, $c);
    }
    
    return array($a,$b,$c);
}

function findJSFile($fname, $default) {
	if (is_readable($fname)) return $fname;
	if (is_readable('js/' . $fname)) return 'js/' . $fname;
	if (is_readable('script/' . $fname)) return 'script/' . $fname;

	if (!isset($_SERVER) || !$_SERVER['DOCUMENT_ROOT']) return $default;
	
	if (is_readable($_SERVER['DOCUMENT_ROOT'] . '/' . $fname)) return '/' . $fname;
	if (is_readable($_SERVER['DOCUMENT_ROOT'] . '/js/' . $fname)) return '/js/' . $fname;
	if (is_readable($_SERVER['DOCUMENT_ROOT'] . '/script/' . $fname)) return '/script/' . $fname;
		
	return $default;
}
